from pychonet.EchonetInstance import EchonetInstance

FAN_SPEED = {
    "auto": 0x41,
    "minimum": 0x31,
    "low": 0x32,
    "medium-low": 0x33,
    "medium": 0x34,
    "medium-high": 0x35,
    "high": 0x36,
    "very-high": 0x37,
    "max": 0x38,
}

PHOTOCATALYST_STATUS = {"yes": 0x41, "no": 0x42}


# ----- Air Cleaner Class -------
# filter change status notify
def _0135E1(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x41: "yes",
        0x42: "no",
    }
    return values.get(op_mode, "invalid_setting")


# air volume 0x41 for auto
def _0135A0(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x41: "auto",
        0x31: "minimum",
        0x32: "low",
        0x33: "medium-low",
        0x34: "medium",
        0x35: "medium-high",
        0x36: "high",
        0x37: "very-high",
        0x38: "max",
    }
    return values.get(op_mode, "invalid_setting")


# cigarette sensor status
def _0135C1(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "yes", 0x42: "no"}
    return values.get(op_mode, "invalid_setting")


# Photocatalyst setting
def _0135C2(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "on", 0x42: "off"}
    return values.get(op_mode, "invalid_setting")


# air pollution status
def _0135C0(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "pollution", 0x42: "fresh"}
    return values.get(op_mode, "invalid_setting")


"""Class for Air Cleaner Objects"""


class HomeAirCleaner(EchonetInstance):

    EPC_FUNCTIONS = {
        0xE1: _0135E1,
        0xA0: _0135A0,
        0xC0: _0135C0,
        0xC1: _0135C1,
        0xC2: _0135C1,
    }

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x01  # Air conditioner-related device group
        self._eojcc = 0x35  # Air Cleaner
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    """
    getFilterChangeStatus get the status of filter status.

    return: A string representing the lock status
    """

    def getFilterChangeStatus(self):
        return self.getMessage(0xE1)

    """
    setFanSpeed set the desired fan speed (e.g Low, Medium, High etc)

    param fans_speed: A string representing the fan speed
    """

    def setFanSpeed(self, fan_speed):
        return self.setMessage(0xA0, FAN_SPEED[fan_speed])

    """
    GetFanSpeed gets the current fan speed (e.g Low, Medium, High etc)
    Refer EPC code 0xA0: ('Air flow rate setting')

    return: A string representing the fan speed
    """

    def getFanSpeed(self):  # 0xA0
        return self.getMessage(0xA0)

    """
    getCigaretteSensorStatus get the status of cigarette sensor status.

    return: A string representing the lock status
    """

    def getCigaretteSensorStatus(self):
        return self.getMessage(0xC1)

    """
    getAirPollutionStatus get the status of air pollution status.

    return: A string representing the lock status
    """

    def getAirPollutionStatus(self):
        return self.getMessage(0xC0)

    """
    getPhotocatalystStatus get the status of Photocatalyst status.

    return: A string representing the lock status
    """

    def getPhotocatalystStatus(self):
        return self.getMessage(0xC2)

    """
    setPhotocatalyst set the Photocatalyst status

    param photocatalyst_status: A string representing the Photocatalyst speed
    """

    def setPhotocatalyst(self, photocatalyst_status):
        return self.setMessage(0xC2, PHOTOCATALYST_STATUS[photocatalyst_status])



VERSION = "2.2.0"
# Echonetlite message format:
#
# - EDT0     |Property value data             |01 ..|01 01 30 01
#  - NUM     |Total number of instances       |01   |1
#  - EOJ     |ECHONET Lite object specificat..|01 ..|01 30 01
#    - EOJX1 |Class group code                |01   |Air conditioner-related device class group
#    - EOJX2 |Class code                      |30   |Home air conditioner class
#    - EOJX3 |Instance code                   |01   |1

ENL_PORT = 3610
ENL_MULTICAST_ADDRESS = "224.0.23.0"
MESSAGE_TIMEOUT = 100


# ------------------------------------------------------------------
# EHD1: ECHONET Lite Header 1
# ---------------------------------------------------------------- */
EHD1 = {0x00: "Not available", 0x10: "Conventional ECHONET Lite Specification"}

# ------------------------------------------------------------------
# EHD1: ECHONET Lite Header 2
# ---------------------------------------------------------------- */
EHD2 = {
    0x81: "Format 1 (specified message format)",
    0x82: "Format 2 (arbitrary message format)",
}

# ------------------------------------------------------------------
# ESV
# ------------------------------------------------------------------
GETC = 0x60
SETC = 0x61
GET = 0x62
INFREQ = 0x63
SETGET = 0x6E
SETRES = 0x71
GETRES = 0x72
INF = 0x73
INFC = 0x74
INFC_RES = 0x7A
SETGET_RES = 0x7E
SETI_SNA = 0x50
SETC_SND = 0x51
GET_SNA = 0x52
INF_SNA = 0x53
SETGET_SNA = 0x5E
INSTANCE_LIST = 0xD6

ESV_CODES = {
    0x60: {
        "name": "GetC",
        "description": "Property value write request (no response required)",
    },
    0x61: {
        "name": "SetC",
        "description": "Property value write request (response required)",
    },
    0x62: {"name": "Get", "description": "Property value read request"},
    0x63: {"name": "INF_REQ", "description": "Property value notification request"},
    0x6E: {"name": "SetGet", "description": "Property value write & read request"},
    0x71: {
        "name": "Set_Res",
        "description": "Property value Property value write response",
    },
    0x72: {"name": "Get_Res", "description": "Property value read response"},
    0x73: {"name": "INF", "description": "Property value notification"},
    0x74: {
        "name": "INFC",
        "description": "Property value notification (response required)",
    },
    0x7A: {"name": "INFC_Res", "description": "Property value notification response"},
    0x7E: {"name": "SetGet_Res", "description": "Property value write & read response"},
    0x50: {
        "name": "SetI_SNA",
        "description": "Property value write request (response not possible)",
    },
    0x51: {
        "name": "SetC_SNA",
        "description": "Property value write request (response not possible)",
    },
    0x52: {
        "name": "Get_SNA",
        "description": "Property value read (response not possible)",
    },
    0x53: {
        "name": "INF_SNA",
        "description": "Property value notification (response not possible)",
    },
    0x5E: {
        "name": "SetGet_SNA",
        "description": "Property value write & read (response not possible)",
    },
}

MANUFACTURERS = {
    0x000001: "Hitachi",
    0x000005: "Sharp",
    0x000006: "Mitsubishi Electric",
    0x000008: "DAIKIN INDUSTRIES",
    0x000009: "NEC",
    0x00000B: "Panasonic",
    0x00000D: "Panasonic System Networks",
    0x000012: "Oi",
    0x000015: "Daikin Systems & Solutions Laboratory",
    0x000016: "Toshiba",
    0x000017: "Toshiba Carrier",
    0x00001B: "Toshiba Lighting & Technology",
    0x000022: "Hitach Appliance",
    0x000023: "NTT Comware",
    0x000025: "LIXIL",
    0x00002C: "AFT",
    0x00002E: "SHIKOKU INSTRUMENTATION",
    0x00002F: "AIPHONE",
    0x000034: "MITSUBISHI ELECTRIC ENGINEERING",
    0x000035: "Toshiba Toko Meter Systems",
    0x000036: "NISSIN SYSTEMS",
    0x000038: "Adosol Nissin",
    0x000039: "Sanden",
    0x00003A: "Sekisui House",
    0x00003B: "Kyocera",
    0x00003C: "Denso",
    0x00003D: "Sumitomo Electric Industries",
    0x00003E: "Sumitomo Electric Networks",
    0x000040: "Hitach Hi-Tech Solutions",
    0x000041: "Enegate",
    0x000043: "Toshiba Digital Media Engineering",
    0x000044: "Hitach Industrial Equipment Systems",
    0x000045: "TIS",
    0x000047: "NTT East",
    0x000048: "Oki Electric Industry",
    0x00004D: "INABA DENKI SANGYO",
    0x00004E: "Fujitsu",
    0x00004F: "Daiwa House Industry",
    0x000050: "TOTO",
    0x000051: "Fuji IT",
    0x000052: "Osaki Electric",
    0x000053: "Ubiquitous",
    0x000054: "Noritz",
    0x000055: "Familynet Japan",
    0x000056: "iND",
    0x000057: "Elii Power",
    0x000058: "Mediotech",
    0x000059: "Rinnai",
    0x00005C: "Transboot",
    0x00005E: "GWSOLAR",
    0x000060: "Sony CSL",
    0x000061: "NTT Data Intellilink",
    0x000063: "Kawamura Electric",
    0x000064: "OMRON",
    0x000067: "CORONA",
    0x000068: "Aisin Seiki",
    0x000069: "Toshiba Lifestyle Products & Services",
    0x00006A: "Okaya",
    0x00006B: "ISB",
    0x00006C: "Nichicon",
    0x00006E: "Soundvision",
    0x00006F: "Buffalo",
    0x000070: "CEC",
    0x000071: "Nihon Sangyo",
    0x000072: "Eneres",
    0x000073: "NEC Platforms",
    0x000075: "RICOH IT Solutions",
    0x000076: "TSP",
    0x000077: "Kanagawa Institute of Technology",
    0x000078: "Hitach Maxell",
    0x000079: "ANRITSU Engineering",
    0x00007A: "Zuken Elmic",
    0x00007C: "Nihon Systemware",
    0x00007E: "SMK",
    0x00007F: "Anritsu Customer Support",
    0x000080: "Tabuchi Electric",
    0x000081: "Iwatsu Electric",
    0x000082: "Purpose",
    0x000083: "Melco Techno Yokohama",
    0x000084: "Rohm",
    0x000085: "Takaoka Toko",
    0x000086: "NTT West",
    0x000087: "IO Data Device",
    0x000088: "Chofu Seisakusho",
    0x000089: "Hokkaido Electric Industries",
    0x00008A: "Fujitsu General",
    0x00008C: "Kyuden Technosystems",
    0x00008D: "NTT",
    0x00008E: "Yamaha",
    0x00008F: "Glamo",
    0x000090: "Fujitsu Component",
    0x000091: "NEC Engineering",
    0x000092: "LSIS",
    0x000093: "Satori Electric",
    0x000095: "Yamato Denki",
    0x000096: "Azbil",
    0x000097: "Future Technology Laboratories",
    0x000099: "Tokyo Electric Power Company Holdings",
    0x00009A: "The Kansai Electric Power Company",
    0x00009B: "Gastar",
    0x00009C: "Diamond Electric Mfg",
    0x00009E: "Yaskawa Electric",
    0x00009F: "GS Yuasa International",
    0x0000A0: "NTT Advance Technology",
    0x0000A1: "Honda R&D",
    0x0000A2: "Renaisance",
    0x0000A3: "Chubu Electric Power Company",
    0x0000A5: "Nichibei",
    0x0000A7: "Jel System",
    0x0000A8: "Smart Power System",
    0x0000A9: "Onkyo Development & Manufacturing",
    0x0000AA: "Ten Feet Wright",
    0x0000AC: "IDEC",
    0x0000AD: "Delta Electronics",
    0x0000AE: "Shikoku Electric Power Company",
    0x0000AF: "Takara Standard",
    0x0000B0: "Naltec",
    0x0000B1: "Internet Initiative Japan",
    0x0000B2: "NF",
    0x0000B3: "Toppers Project",
    0x0000B4: "4R Energy",
    0x0000B5: "The Chugoku Electric Power Company",
    0x0000B6: "Bunka Shutter",
    0x0000B7: "Nitto Kogyo",
    0x0000B8: "Hokkaido Electric Power Company",
    0x0000B9: "Energy Demand Side Service Development",
    0x0000BA: "Sankyo Tateyama",
    0x0000BB: "Hokuriku Electric Power Company",
    0x0000BC: "Tohoku Electric Power Company",
    0x0000BE: "Denken",
    0x0000BF: "Kyushyu Electric Power Company",
    0x0000C1: "Tsuken Electric Ind",
    0x0000C2: "Tohoku Electric Meter Industry",
    0x0000C3: "JEMIC",
    0x0000C5: "Sanwa Shutter",
    0x0000C9: "Kikusui",
    0x0000CA: "JSP",
    0x0000CB: "Fuji Electric",
    0x0000CC: "Hitach-Johnson Controls Air Conditioning",
    0x0000CD: "Toclas",
    0x0000CE: "Shindengen Electric Manufacturing",
    0x0000D0: "Tsubamoto Chain",
    0x0000D1: "Takebishi",
    0x0000D2: "Chofukosan",
    0x0000D4: "Murata Manufacturing",
    0x0000D5: "Chosyu Industry",
    0x0000D6: "Sassor",
    0x0000D7: "Kaga Electronics",
    0x0000D8: "Osaki Datatech",
    0x0000D9: "Toshiba IT Control Systems",
    0x0000DA: "Panasonic Commercial Equipment Systems",
    0x0000DB: "Santech Power Japan",
    0x0000DC: "Nihon Techno",
    0x0000DD: "EneStone",
    0x0000DE: "Fuji Xerox Hokkaido",
    0x0000E0: "Looop",
    0x0000E1: "Softbank",
    0x0000E2: "NextDrive",
    0x0000E3: "DDL",
    0x0000E4: "Technoeyes",
    0x0000E5: "Hitach Power Solutions",
    0x0000E6: "Hokkaido Electrical Safty Services Foundation",
    0x0000E7: "Infometis",
    0x0000E8: "Koizumi Lighting Technology",
    0x0000E9: "NTT Smile Energy",
    0x0000EA: "SI Solar",
    0x0000EB: "Nichicon Kameoka",
    0x0000EC: "Toshiba Energy Systems & Solutions",
    0x0000ED: "Infini",
    0x0000EE: "Tessera Technology",
    0x0000EF: "Toyoda industories",
    0x0000F0: "Kaneka",
    0x0000F1: "Laplace Systems",
    0x0000F2: "Energy Solutions",
    0x0000F3: "Energy Gateway",
    0x0000F4: "Denso Aircool",
    0x0000F6: "Field Logic",
    0x0000F7: "JCity",
    0x0000F8: "Avenir",
    0x0000F9: "Toho Electronics",
    0x0000FA: "Plat Home",
    0x0000FB: "CICO",
    0x0000FC: "Fuji Industrial",
    0xFFFFFE: "Undefined",
    0xFFFFFF: "Experimental",
}

ENL_ON = 0x30
ENL_OFF = 0x31
ENL_STATUS = 0x80
ENL_UID = 0x83
ENL_INSTANTANEOUS_POWER = 0x84
ENL_CUMULATIVE_POWER = 0x85
ENL_MANUFACTURER = 0x8A
ENL_CUMULATIVE_RUNTIME = 0x9A
ENL_SETMAP = 0x9E
ENL_GETMAP = 0x9F



# ------------------------------------------------------------------
# EOJX1: Group Name
# ---------------------------------------------------------------- */
EOJX_GROUP = {
    0x00: "Sensor-related device group",
    0x01: "Air conditioner-related device group",
    0x02: "Housing/facility-related device group",
    0x03: "Cooking/housework-related device group",
    0x04: "Health-related device group",
    0x05: "Management/control-related device group",
    0x06: "AV-related device group",
    0x0E: "Profile group",
    0x0F: "User definition group",
}

# ------------------------------------------------------------------
# EOJX2: Name
# ---------------------------------------------------------------- */
EOJX_CLASS = {
    0x00: {  # Sensor-related device group
        0x01: "Gas leak sensor",
        0x02: "Crime prevention sensor",
        0x03: "Emergency button",
        0x04: "First-aid sensor",
        0x05: "Earthquake sensor",
        0x06: "Electric leak sensor",
        0x07: "Human detection sensor",
        0x08: "Visitor sensor",
        0x09: "Call sensor",
        0x0A: "Condensation sensor",
        0x0B: "Air pollution sensor",
        0x0C: "Oxygen sensor",
        0x0D: "Illuminance sensor",
        0x0E: "Sound sensor",
        0x0F: "Mailing sensor",
        0x10: "Weight sensor",
        0x11: "Temperature sensor",
        0x12: "Humidity sensor",
        0x13: "Rain sensor",
        0x14: "Water level sensor",
        0x15: "Bath water level sensor",
        0x16: "Bath heating status sensor",
        0x17: "Water leak sensor",
        0x18: "Water overflow sensor",
        0x19: "Fire sensor",
        0x1A: "Cigarette smoke sensor",
        0x1B: "CO2 sensor",
        0x1C: "Gas sensor",
        0x1D: "VOC sensor",
        0x1E: "Differential pressure sensor",
        0x1F: "Air speed sensor",
        0x20: "Odor sensor",
        0x21: "Flame sensor",
        0x22: "Electric energy sensor",
        0x23: "Current value sensor",
        0x24: "Daylight sensor",
        0x25: "Water flow rate sensor",
        0x26: "Micromotion sensor",
        0x27: "Passage sensor",
        0x28: "Bed presence sensor",
        0x29: "Open/close sensor",
        0x2A: "Activity amount sensor",
        0x2B: "Human body location sensor",
        0x2C: "Snow sensor",
        0x2D: "Air pressure sensor",
    },
    0x01: {  # / Air conditioner-related device group
        0x30: "Home air conditioner",
        0x31: "Cold blaster",
        0x32: "Electric fan",
        0x33: "Ventilation fan",
        0x34: "Air conditioner ventilation fan",
        0x35: "Air cleaner",
        0x36: "Cold blast fan",
        0x37: "Circulator",
        0x38: "Dehumidifier",
        0x39: "Humidifier",
        0x3A: "Ceiling fan",
        0x3B: "Electric Kotatsu",
        0x3C: "Electric heating pad",
        0x3D: "Electric blanket",
        0x3E: "Space heater",
        0x3F: "Panel heater",
        0x40: "Electric carpet",
        0x41: "Floor heater",
        0x42: "Electric heater",
        0x43: "Fan heater",
        0x44: "Battery charger",
        0x45: "Package-type commercial air conditioner (indoor unit)",
        0x46: "Package-type commercial air conditioner (outdoor unit)",
        0x47: "Package-type commercial air conditioner thermal storage unit",
        0x48: "Commercial fan coil unit",
        0x49: "Commercial air conditioner cold source (chiller)",
        0x50: "Commercial air conditioning hot source (boiler)",
        0x51: "Air-conditioning VAV for commercial applications",
        0x52: "Air handling unit (air-conditioning) for commercial applications",
        0x53: "Unit-cooler",
        0x54: "Condensing unit for commercial applications",
        0x55: "Electric storage heater",
    },
    0x02: {  # Housing/facility-related device group
        0x60: "Electrically operated blind/shade",
        0x61: "Electrically operated shutter",
        0x62: "Electrically operated curtain",
        0x63: "Electrically operated rain sliding door/shutter",
        0x64: "Electrically operated gate",
        0x65: "Electrically operated window",
        0x66: "Automatically operated entrance door/sliding door",
        0x67: "Garden sprinkler",
        0x68: "Fire sprinkler",
        0x69: "Fountain",
        0x6A: "Instantaneous water heater",
        0x6B: "Electric water heater",
        0x6C: "Solar water heater",
        0x6D: "Circulation pump",
        0x6E: "Bidet-equipped toilet (with electrically warmed seat)",
        0x6F: "Electric lock",
        0x70: "Gas line valve",
        0x71: "Home sauna",
        0x72: "Hot water generator",
        0x73: "Bathroom dryer",
        0x74: "Home elevator",
        0x75: "Electrically operated room divider",
        0x76: "Horizontal transfer",
        0x77: "Electrically operated clothes-drying pole",
        0x78: "Septic tank",
        0x79: "Home solar power generation",
        0x7A: "Cold/hot water heat source equipment",
        0x7B: "Floor heater",
        0x7C: "Fuel cell",
        0x7D: "Storage battery",
        0x7E: "Electric vehicle charger/discharger",
        0x7F: "Engine cogeneration",
        0x80: "Electric energy meter",
        0x81: "Water flow meter",
        0x82: "Gas meter",
        0x83: "LP gas meter",
        0x84: "Clock",
        0x85: "Automatic door",
        0x86: "Commercial elevator",
        0x87: "Distribution panel metering",
        0x88: "Low voltage smart electric energy meter",
        0x89: "Smart gas meter",
        0x8A: "High voltage smart electric energy meter",
        0x90: "General lighting",
        0x91: "Single function lighting",
        0x99: "Emergency lighting",
        0x9D: "Equipment light",
        0xA0: "Buzzer",
        0xA2: "Household small wind turbine",
        0xA3: "Lighting system",
        0xA4: "Extended Lighting system",
        0xA5: "Multiple Input PCS",
        0xA6: "Hybrid Water Heater"
    },
    0x03: {  # Cooking/housework-related device group
        0xB0: "Coffee machine",
        0xB1: "Coffee mill",
        0xB2: "Electric hot water pot (Electric thermos)",
        0xB3: "Electric stove",
        0xB4: "Toaster",
        0xB5: "Juicer, food mixer",
        0xB6: "Food processor",
        0xB7: "Refrigerator",
        0xB8: "Combination microwave oven (Electronic oven)",
        0xB9: "Cooking heater",
        0xBA: "Oven",
        0xBB: "Raice cooker",
        0xBC: "Electronic jar",
        0xBD: "Dish washer",
        0xBE: "Dish dryer",
        0xBF: "Electric rice card cooker",
        0xC0: "Keep-warm machine",
        0xC1: "Race mill",
        0xC2: "Automatic bread cooker",
        0xC3: "Slow cooker",
        0xC4: "Electric pickles cooker",
        0xC5: "Washing machine",
        0xC6: "Clothes dryer",
        0xC7: "Electric iron",
        0xC8: "Trouser press",
        0xC9: "Futon dryer",
        0xCA: "Small article, shoes dryer",
        0xCB: "Electric vacuum cleaner (including central vacuum cleaner)",
        0xCC: "Disposer",
        0xCD: "Electric mosquito catcher",
        0xCE: "Commercial show case",
        0xCF: "Commercial refrigerator",
        0xD0: "Commercial hot case",
        0xD1: "Commercial fryer",
        0xD2: "Commercial microwave oven",
        0xD3: "Washer and dryer",
        0xD4: "Commercial show case outdoor unit",
    },
    0x04: {  # Health-related device group
        0x01: "Weighing machine",
        0x02: "Clinical thermometer",
        0x03: "Blood pressure meter",
        0x04: "Blood suger meter",
        0x05: "Body fat meter",
    },
    0x05: {  # Management/control-related device group
        0xFC: "Secure communication shared key setup node",
        0xFD: "Switch (supporting JEM-A/HA terminals)",
        0xFE: "Portable (mobile) terminal",
        0xFF: "Controller",
    },
    0x06: {  # AV-related device group
        0x01: "Display",
        0x02: "Television",
        0x03: "Audio",
        0x04: "Network camera",
    },
    0x0E: {0xF0: "Node profile"},  # Profile group
}



from pychonet.EchonetInstance import EchonetInstance
from pychonet.lib.epc_functions import _int, _signed_int

def _0287C2(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x00: 1,
              0x01: 0.1,
              0x02: 0.01,
              0x03: 0.001,
              0x04: 0.0001,
              0x0A: 10,
              0x0B: 100,
              0x0C: 1000,
              0x0D: 10000}
    return values.get(op_mode, None)

# def _0287xx(edt):
#     return "Not implemented"
#
# def _0287D0EF(edt):
#     return "Measurement channels not implemented"

def _0287C7(edt):
    r_phase = float(int.from_bytes(edt[0:2], "big", signed=True)) / 10  # R Phase
    t_phase = float(int.from_bytes(edt[2:4], "big", signed=True)) / 10  # T Phase
    if t_phase == 3276.6:
        t_phase = None
    return {"r_phase_amperes": r_phase, "t_phase_amperes": t_phase}

def _0287C8(edt):
    r_sn = float(int.from_bytes(edt[0:2], "big", signed=True)) / 10  # R Phase
    sn_t = float(int.from_bytes(edt[2:4], "big", signed=True)) / 10  # T Phase
    if sn_t == 6553.4:
        sn_t = None
    return {"r_sn_voltage": r_sn, "sn_t_voltage": sn_t}

class DistributionPanelMeter(EchonetInstance):
    EPC_FUNCTIONS = {
        0xC0: _int,        # "Measured cumulative amount of electric energy (normal direction)"
        0xC1: _int,        # "Measured cumulative amount of electric energy (reverse direction)"
        0xC2: _0287C2,     # "Unit for cumulative amounts of electric energy"
#       0xC3: _0287xx,     # "Historical data of measured cumulative amounts of electric energy (normal direction)"
#       0xC4: _0287xx,     # "Historical data of measured cumulative amounts of electric energy (reverse direction)"
        0xC5: _int,        # "Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved"
        0xC6: _signed_int, # "Measured instantaneous amount of electric energy"
        0xC7: _0287C7,     # "Measured instantaneous currents"
        0xC8: _0287C8,     # "Measured instantaneous voltages"
#       0xD0: _0287D0EF,   # "Measurement channel 1"
#       0xD1: _0287D0EF,   # "Measurement channel 2"
#       0xD2: _0287D0EF,   # "Measurement channel 3"
#       0xD3: _0287D0EF,   # "Measurement channel 4"
#       0xD4: _0287D0EF,   # "Measurement channel 5"
#       0xD5: _0287D0EF,   # "Measurement channel 6"
#       0xD6: _0287D0EF,   # "Measurement channel 7"
#       0xD7: _0287D0EF,   # "Measurement channel 8"
#       0xD8: _0287D0EF,   # "Measurement channel 9"
#       0xD9: _0287D0EF,   # "Measurement channel 10"
#       0xDA: _0287D0EF,   # "Measurement channel 11"
#       0xDB: _0287D0EF,   # "Measurement channel 12"
#       0xDC: _0287D0EF,   # "Measurement channel 13"
#       0xDD: _0287D0EF,   # "Measurement channel 14"
#       0xDE: _0287D0EF,   # "Measurement channel 15"
#       0xDF: _0287D0EF,   # "Measurement channel 16"
#       0xE0: _0287D0EF,   # "Measurement channel 17"
#       0xE1: _0287D0EF,   # "Measurement channel 18"
#       0xE2: _0287D0EF,   # "Measurement channel 19"
#       0xE3: _0287D0EF,   # "Measurement channel 20"
#       0xE4: _0287D0EF,   # "Measurement channel 21"
#       0xE5: _0287D0EF,   # "Measurement channel 22"
#       0xE6: _0287D0EF,   # "Measurement channel 23"
#       0xE7: _0287D0EF,   # "Measurement channel 24"
#       0xE8: _0287D0EF,   # "Measurement channel 25"
#       0xE9: _0287D0EF,   # "Measurement channel 26"
#       0xEA: _0287D0EF,   # "Measurement channel 27"
#       0xEB: _0287D0EF,   # "Measurement channel 28"
#       0xEC: _0287D0EF,   # "Measurement channel 29"
#       0xED: _0287D0EF,   # "Measurement channel 30"
#       0xEE: _0287D0EF,   # "Measurement channel 31"
#       0xEF: _0287D0EF,   # "Measurement channel 32"
        0xB0: _int,        # "Master rated capacity"
        0xB1: _int,        # "Number of measurement channels (simplex)"
#       0xB2: _0287xx,     # "Channel range specification for cumulative amount of electric power consumption measurement (simplex)"
#       0xB3: _0287xx,     # "Measured cumulative amount of electric power consumption list (simplex)"
#       0xB4: _0287xx,     # "Channel range specification for instantaneous current measurement (simplex)"
#       0xB5: _0287xx,     # "Measured instantaneous current list (simplex)"
#       0xB6: _0287xx,     # "Channel range specification for instantaneous power consumption measurement (simplex)"
#       0xB7: _0287xx,     # "Measured instantaneous power consumption list (simplex)"
        0xB8: _int,        # "Number of measurement channels (duplex)"
#       0xB9: _0287xx,     # "Channel range specification for cumulative amount of electric power consumption measurement (duplex)"
#       0xBA: _0287xx,     # "Measured cumulative amount of electric power consumption list (duplex)"
#       0xBB: _0287xx,     # "Channel range specification for instantaneous current measurement (duplex)"
#       0xBC: _0287xx,     # "Measured instantaneous current list (duplex)"
#       0xBD: _0287xx,     # "Channel range specification for instantaneous power consumption measurement (duplex)"
#       0xBE: _0287xx,     # "Measured instantaneous power consumption list (duplex)"
    }

    def __init__(self, host, api_connector, instance=0x1):
        self._eojgc = 0x02
        self._eojcc = 0x87
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )



from pychonet.EchonetInstance import EchonetInstance

MEASURED_TEMP = 0xE0

# ----- Tempereature Sensor -------


def _0011E0(edt):
    return float(int.from_bytes(edt, "big")) / 10


class TemperatureSensor(EchonetInstance):
    EPC_FUNCTIONS = {
        0xE0: _0011E0,
    }

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x00
        self._eojcc = 0x11
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    def getMeasuredTemperature(self):
        return self.getMessage(MEASURED_TEMP)



from datetime import datetime, timezone

from pychonet.lib.const import MANUFACTURERS


# ------------ EPC GENERIC FUNCTIONS -------
def _int(edt):  # unsigned int
    return int.from_bytes(edt, "big")


def _signed_int(edt):  # signed ints
    return int.from_bytes(edt, byteorder="big", signed=True)


# Check status of Echonnet Instance
# ----------------- EPC SUPER FUNCTIONS -----------------------------
def _0080(edt):
    ops_value = int.from_bytes(edt, "big")
    return "On" if ops_value == 0x30 else "Off"


def _009X(edt):
    payload = []
    if len(edt) < 17:
        for i in range(1, len(edt)):
            payload.append(edt[i])
        return payload

    for i in range(1, len(edt)):
        code = i - 1
        binary = "{0:08b}".format(edt[i])[::-1]
        for j in range(0, 8):
            if binary[j] == "1":
                EPC = (j + 8) * 0x10 + code
                payload.append(EPC)
    return payload


def _0083(edt):
    if edt is not None:
        if len(edt) > 1:
            ops_value = edt[1:].hex()
        else:
            ops_value = None
        return ops_value
    return None


def _008A(edt):  # manufacturer
    id = int.from_bytes(edt, "big")
    if id in MANUFACTURERS.keys():
        return MANUFACTURERS[id]
    return id


def _009A(edt):  # cumulative runtime
    if len(edt) > 1:
        value = int.from_bytes(edt[1:], "big")
        time_period_multiplier = 0
        if edt[0] == 0x41:
            time_period_multiplier = 1  # measurement is in seconds
        elif edt[0] == 0x42:
            time_period_multiplier = 60  # measurement is in minutes
        elif edt[0] == 0x43:
            time_period_multiplier = 3600  # measurement is in hours
        elif edt[0] == 0x44:
            time_period_multiplier = 3600 * 24  # measurement is in days
        return value * time_period_multiplier
    return None


EPC_SUPER_FUNCTIONS = {
    0x80: _0080,
    0x83: _0083,
    0x84: _int,
    0x85: _int,
    0x8A: _008A,
    0x9A: _009A,
    0x9E: _009X,
    0x9F: _009X,
}


# ------- EPC FUNCTIONS -------------------------------------------------
# TODO - Move these to their classes
# -----------------------------------------------------------------------


# --- Low voltage smart meter class


def _0288E1(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x00: 1,
        0x01: 0.1,
        0x02: 0.01,
        0x03: 0.001,
        0x04: 0.0001,
        0x0A: 10,
        0x0B: 100,
        0x0C: 1000,
        0x0D: 10000,
    }
    return values.get(op_mode, "invalid_setting")


# ----- Low voltage smart electric energy meter -------
def _0288E7(edt):
    value = int.from_bytes(edt, "big", signed=True)
    return value


def _0288E8(edt):
    r_phase = float(int.from_bytes(edt[0:2], "big", signed=True)) / 10  # R Phase
    t_phase = float(int.from_bytes(edt[2:4], "big", signed=True)) / 10  # T Phase
    return {"r_phase_amps": r_phase, "t_phase_amps": t_phase}


def _0288EA(edt):
    print(edt)
    year = int.from_bytes(edt[0:2], "big")
    month = int.from_bytes(edt[2:3], "big")
    day = int.from_bytes(edt[3:4], "big")
    hour = int.from_bytes(edt[4:5], "big")
    minute = int.from_bytes(edt[5:6], "big")
    second = int.from_bytes(edt[6:7], "big")
    culmative = int.from_bytes(edt[7:], "big")
    time = datetime(year, month, day, hour, minute, second)
    return {
        "time": datetime(year, month, day, hour, minute, second).isoformat(),
        "culmative_value": culmative,
    }



from pychonet.EchonetInstance import EchonetInstance


# ----- Electric Lock Class -------
def _026FEX(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "lock", 0x42: "unlock"}
    return values.get(op_mode, "invalid_setting")


def _026FE3(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "open", 0x42: "closed"}
    return values.get(op_mode, "invalid_setting")


def _026FE4(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "occupant", 0x42: "non-occupant"}
    return values.get(op_mode, "invalid_setting")


def _026FE5(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x40: "normal",
        0x41: "break_open",
        0x42: "door_open",
        0x43: "maunal_unlocked",
        0x44: "tampered",
    }
    return values.get(op_mode, "invalid_setting")


def _026FE6(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "on", 0x42: "off"}
    return values.get(op_mode, "invalid_setting")


def _026FE7(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "normal", 0x42: "battery_replacement_notification"}
    return values.get(op_mode, "invalid_setting")


"""Class for Electric Lock Objects"""


class ElectricLock(EchonetInstance):

    EPC_FUNCTIONS = {
        0xE1: _026FEX,
        0xE2: _026FEX,
        0xE3: _026FE3,
        0xE4: _026FE4,
        0xE5: _026FE5,
        0xE6: _026FE6,
        0xE7: _026FE7,
    }

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x02  # Housing/facility-related device group
        self._eojcc = 0x6F  # Electric Lock
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    """
    getLockOneStatus get the status of lock one.

    return: A string representing the lock status
    """

    def getLockOneStatus(self):
        return self.getMessage(0xE1)



import asyncio

from pychonet.lib.const import (ENL_GETMAP, ENL_MANUFACTURER, ENL_PORT,
                                ENL_SETMAP, ENL_UID, GET, MESSAGE_TIMEOUT)
from pychonet.lib.epc_functions import EPC_SUPER_FUNCTIONS
from pychonet.lib.functions import TIDError, buildEchonetMsg, decodeEchonetMsg


class ECHONETAPIClient:
    def __init__(self, server, loop):
        self._server = server
        self._loop = loop
        self._server.subscribe(self.echonetMessageReceived)
        self._state = {}
        self._next_tx_tid = 0x0000
        self._message_list = []
        self._message_timeout = MESSAGE_TIMEOUT
        self._debug_flag = False

    async def echonetMessageReceived(self, raw_data, addr):
        host = addr[0]
        processed_data = decodeEchonetMsg(raw_data)
        if self._debug_flag:
            print(f"Echonet Message Received - Processed data is {processed_data}")
        # if we get duplicate packets that have already been processed then dont worry about the message list.
        # but still process them regardless.
        if processed_data["TID"] in self._message_list:
            self._message_list.remove(processed_data["TID"])
        # handle discovery message response
        for opc in processed_data["OPC"]:
            seojgc = processed_data["SEOJGC"]
            seojcc = processed_data["SEOJCC"]
            seojci = processed_data["SEOJCI"]
            epc = opc["EPC"]
            if seojgc == 14 and seojcc == 240 and epc == 0xD6:
                await self.process_discovery_data(host, opc)
            else:
                if epc == ENL_SETMAP:
                    map = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = map
                elif epc == ENL_GETMAP:
                    map = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = map
                elif epc in (ENL_UID, ENL_MANUFACTURER):
                    self._state[host]["instances"][seojgc][seojcc][seojci][
                        epc
                    ] = EPC_SUPER_FUNCTIONS[epc](opc["EDT"])
                else:
                    self._state[host]["instances"][seojgc][seojcc][seojci][epc] = opc[
                        "EDT"
                    ]

    async def discover(self, host="224.0.23.0"):
        return await self.echonetMessage(host, 0x0E, 0xF0, 0x00, GET, [{"EPC": 0xD6}])

    async def echonetMessage(self, host, deojgc, deojcc, deojci, esv, opc):
        payload = None
        message_array = {
            "DEOJGC": deojgc,
            "DEOJCC": deojcc,
            "DEOJCI": deojci,
            "ESV": esv,
            "OPC": opc,
        }
        if host not in list(self._state.keys()):
            self._state[host] = {"instances": {}}

        self._next_tx_tid += 1
        tx_tid = self._next_tx_tid
        message_array["TID"] = tx_tid
        try:
            payload = buildEchonetMsg(message_array)
        except TIDError:  # Quashing the rollover bug hopefully once and for all...
            self._next_tx_tid = 1
            tx_tid = self._next_tx_tid
            message_array["TID"] = tx_tid
            payload = buildEchonetMsg(message_array)

        self._message_list.append(tx_tid)
        self._server.send(payload, (host, ENL_PORT))
        for x in range(0, self._message_timeout):
            await asyncio.sleep(0.01)
            # if tx_tid is not in message list then the message listener has received the message
            if tx_tid not in self._message_list:
                # transaction sucessful remove from list
                return True
        return False

    async def getAllPropertyMaps(self, host, eojgc, eojcc, eojci):
        return await self.echonetMessage(
            host, eojgc, eojcc, eojci, GET, [{"EPC": ENL_GETMAP}, {"EPC": ENL_SETMAP}]
        )

    async def getIdentificationInformation(self, host, eojgc, eojcc, eojci):
        return await self.echonetMessage(
            host,
            eojgc,
            eojcc,
            eojci,
            GET,
            [{"EPC": ENL_UID}, {"EPC": ENL_MANUFACTURER}],
        )

    async def process_discovery_data(self, host, opc_data):
        if "discovered" not in self._state[host]:
            edt = bytearray(opc_data["EDT"])
            # 1st byte: Total number of instances
            # 2nd to 253rd bytes: ECHONET object codes (EOJ3 bytes) enumerated
            edtnum = bytearray(edt)[0]
            for x in range(edtnum):
                eojgc = bytearray(edt)[1 + (3 * x)]
                eojcc = bytearray(edt)[2 + (3 * x)]
                eojci = bytearray(edt)[3 + (3 * x)]

                # populate state table
                if eojgc not in list(self._state[host]["instances"].keys()):
                    self._state[host]["instances"].update({eojgc: {}})
                if eojcc not in list(self._state[host]["instances"][eojgc].keys()):
                    self._state[host]["instances"][eojgc].update({eojcc: {}})
                if eojci not in list(
                    self._state[host]["instances"][eojgc][eojcc].keys()
                ):
                    self._state[host]["instances"][eojgc][eojcc][eojci] = {}
                    self._state[host]["instances"][eojgc][eojcc][eojci].update(
                        {ENL_SETMAP: []}
                    )
                    self._state[host]["instances"][eojgc][eojcc][eojci].update(
                        {ENL_GETMAP: []}
                    )
            self._state[host]["discovered"] = True



from pychonet.EchonetInstance import EchonetInstance


class StorageBattery(EchonetInstance):

    WORKING_OPERATION_STATES = {
        0x40: "Other",
        0x41: "Rapid charging",
        0x42: "Charging",
        0x43: "Discharging",
        0x44: "Standby",
        0x45: "Test",
        0x46: "Automatic",
        0x48: "Restart",
        0x49: "Effective capacity recalculation processing",
    }

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x02
        self._eojcc = 0x7D
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    def getRemainingStoredElectricity3(self):
        return self.getMessage(0xE4)

    def getWorkingOperationStatus(self):
        return self.getMessage(0xCF)



from pychonet.EchonetInstance import EchonetInstance

ENL_OPENSTATE = 0xE0


def _0260EO(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "open", 0x42: "close", 0x43: "stop"}
    return values.get(op_mode, "invalid_setting")


# TODO - complete class definitions
# 			0x80: 'Operation status',
# 			0x89: 'Fault description (Recoverable faults)',
# 			0x90: 'Timer operation setting',
# 			0xC2: 'Wind detection status',
# 			0xC3: 'Sunlight detection status',
# 			0xD0: 'Opening (extension) speed setting',
# 			0xD1: 'Closing (retraction) speed setting',
# 			0xD2: 'Operation time',
# 			0xD4: 'Automatic operation setting',
# 			0xE1: 'Degree-of-opening level',
# 			0xE2: 'Shade angle setting ',
# 			0xE3: 'Open/close (extension/retraction) speed',
# 			0xE5: 'Electric lock setting',
# 			0xE8: 'Remote operation setting status',
# 			0xE9: 'Selective opening (extension) operation setting',
# 			0xEA: 'Open/closed (extended/retracted) status',
# 			0xEE: 'One-time opening (extension) speed setting',
# 			0xEF: 'One-time closing (retraction) speed setting'


"""Class for Electric Blind/Shade Objects"""


class ElectricBlind(EchonetInstance):

    EPC_FUNCTIONS = {0xE0: _0260EO}

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x02  # Housing/facility-related device group
        self._eojcc = 0x60  # Electrically operated blind/shade
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    """
    getOpenCloseSetting get the status of the blind.

    return: A string representing the blind/shade state
    """

    def getOpenCloseSetting(self):
        return self.getMessage(ENL_OPENSTATE)



from pychonet.echonetapiclient import ECHONETAPIClient  # noqa
from pychonet.lib.eojx import EOJX_CLASS

from .EchonetInstance import EchonetInstance
from .ElectricBlind import ElectricBlind
from .ElectricLock import ElectricLock
from .ElectricVehicleCharger import ElectricVehicleCharger
from .GeneralLighting import GeneralLighting
from .HomeAirCleaner import HomeAirCleaner
from .HomeAirConditioner import HomeAirConditioner
from .HomeSolarPower import HomeSolarPower
from .StorageBattery import StorageBattery
from .TemperatureSensor import TemperatureSensor
from .DistributionPanelMeter import DistributionPanelMeter
from .HybridWaterHeater import HybridWaterHeater

def Factory(host, server, eojgc, eojcc, eojci=0x01):
    instance = None
    if eojgc in EOJX_CLASS:
        if eojcc in EOJX_CLASS[eojgc]:
            instance = EOJX_CLASS[eojgc][eojcc]

    """Factory Method"""
    # TODO - probably a much cleaner way of doing this.
    instances = {
        "Home air conditioner": HomeAirConditioner,
        "Home solar power generation": HomeSolarPower,
        "Distribution panel metering": DistributionPanelMeter,
        "Electric vehicle charger/discharger": ElectricVehicleCharger,
        "Temperature sensor": TemperatureSensor,
        "Storage Battery": StorageBattery,
        "Electrically operated blind/shade": ElectricBlind,
        "General lighting": GeneralLighting,
        "Electric Lock": ElectricLock,
        "Air cleaner": HomeAirCleaner,
        "Hybrid Water Heater": HybridWaterHeater,
        None: None,
    }
    instance_object = instances.get(instance, None)
    if instance_object is not None:
        return instance_object(host, server, eojci)

    return EchonetInstance(host, eojgc, eojcc, eojci, server)



from pychonet.EchonetInstance import EchonetInstance

ENL_STATUS = 0x80
ENL_BRIGHTNESS = 0xB0
ENL_COLOR_TEMP = 0xB1

# ----- General lighting class -------


def _0290B1(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x40: "other",
        0x41: "incandescent_lamp_color",
        0x42: "white",
        0x43: "daylight_white",
        0x44: "daylight_color",
    }
    return values.get(op_mode, "invalid_setting")


# TODO - implemenet FUNCTIONS
# 	0xB0: 'Illuminance level',
# 	0xB2: 'Illuminance level step setting',
# 	0xB3: 'Light color step setting',
# 	0xB4: 'Maximum specifiable values',
# 	0xB5: 'Maximum value of settable level for night lighting',
# 	0xB6: 'Lighting mode setting',
# 	0xB7: 'Illuminance level setting for main lighting',
# 	0xB8: 'Illuminance level step setting for main lighting',
# 	0xB9: 'Illuminance level setting for night lighting',
# 	0xBA: 'Illuminance level step setting for night lighting',
# 	0xBB: 'Light color setting for main lighting',
# 	0xBC: 'Light color level step setting for main lighting',
# 	0xBD: 'Light color setting for night lighting',
# 	0xBE: 'Light color level step setting for night lighting',
# 	0xBF: 'Lighting mode status in auto mode',
# 	0xC0: 'RGB setting for color lighting',
# 	0x90: 'ON timer reservation setting',
# 	0x91: 'ON timer setting',
# 	0x94: 'OFF timer reservation setting',
# 	0x95: 'OFF timer setting'


"""Class for General Lighting Objects"""


class GeneralLighting(EchonetInstance):

    EPC_FUNCTIONS = {0xB1: _0290B1}

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x02  # Housing/facility-related device group
        self._eojcc = 0x90  # General Lighting class
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    """
    getBrightness get the brightness that has been set in the light

    return: A string representing the configured brightness.
    """

    def getBrightness(self):
        return self.getMessage(ENL_BRIGHTNESS)  # ['brightness']

    """
    setBrightness set the temperature of the light

    param temperature: A string representing the desired temperature.
    """

    def setBrightness(self, brightness):
        return self.setMessage(ENL_BRIGHTNESS, int(brightness))

    """
    getColorTemp get the color temperature that has been set in the light

    return: A string representing the configured color temperature. # 68 67 66 64 65 coolest to warmest
    """

    def getColorTemperature(self):
        return self.getMessage(ENL_COLOR_TEMP)  # ['color_temperature']

    """
    setColorTemperature set the temperature of the light

    param temperature: A string representing the desired temperature. # 68 67 66 64 65 coolest to warmest
    """

    def setColorTemperature(self, color_temperature):
        return self.setMessage(ENL_COLOR_TEMP, color_temperature)



from pychonet.EchonetInstance import EchonetInstance
from pychonet.lib.epc_functions import _int, _signed_int



# 0xB0 - Automatic water heating setting
def _02A6B0(edt):
    op_mode = int.from_bytes(edt, "big")
    AUTOMATIC_WATER_HEATING_STATES = {
        0x41: "Automatic water heating",
        0x42: "Manual water heating",
        0x43: "Water heating manual stop",
    }
    return AUTOMATIC_WATER_HEATING_STATES.get(op_mode, "Invalid setting")

# 0xB2 - Water heating status
# 0xB3 - Heater status
def _02A6B2B3(edt):
    op_mode = int.from_bytes(edt, "big")
    WATER_HEATING_STATUS = {
        0x41: "Water is heating",
        0x42: "Water is not heating",
    }
    return WATER_HEATING_STATUS.get(op_mode, "Invalid setting")


# 0xB6 - Hot water supply mode setting for auxiliary heat source machine
# 0xB7 - Heater mode setting for auxiliary heat source machine
def _02A6B6B7(edt):
    op_mode = int.from_bytes(edt, "big")
    AUX_SETTING_STATE = {
        0x41: "Set",
        0x42: "No setting",
    }
    return AUX_SETTING_STATE.get(op_mode, "Invalid setting")

# 0xB8 - Linkage mode setting for solar power generation
def _02A6B8(edt):
    op_mode = int.from_bytes(edt, "big")
    SOLAR_POWER_LINKAGE = {
        0x41: "Mode off",
        0x42: "Household consumption",
        0x43: "Prioritizing electricity sales",
        0x44: "Economic efficiency"
    }
    return SOLAR_POWER_LINKAGE.get(op_mode, "Invalid setting")

# 0xB9 - Solar power generations utilization time
def _02A6B9(edt):
    start_hh = float(int.from_bytes(edt[0:1], "big"))
    start_mm = float(int.from_bytes(edt[1:2], "big"))
    end_hh = float(int.from_bytes(edt[2:3], "big"))
    end_mm = float(int.from_bytes(edt[3:4], "big"))
    return { "start_time": f"{start_hh}:{start_mm}", "end_time": f"{end_hh}:{end_mm}" }

# 0xC3 - Hot water supply status
def _02A6C3(edt):
    op_mode = int.from_bytes(edt, "big")
    HOT_WATER_SUPPLY_STATUS = {
        0x41: "Supplying hot water",
        0x42: "Not supplying hot water",
    }
    return HOT_WATER_SUPPLY_STATUS.get(op_mode, "Invalid setting")

# 0xE1 - Measured amount of hot water remaining in tank
# 0xE2 - Tank Capacity

class HybridWaterHeater(EchonetInstance):

    EPC_FUNCTIONS = {
        0xB0: _02A6B0,
        0xB2: _02A6B2B3,
        0xB3: _02A6B2B3,
        0xB6: _02A6B6B7,
        0xB7: _02A6B6B7,
        0xB8: _02A6B8,
        0xB9: _02A6B9,
        0xC3: _02A6C3,
        0xE1: _int,
        0xE2: _int
    }

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x02
        self._eojcc = 0xA6
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )



from pychonet.EchonetInstance import EchonetInstance
from pychonet.lib.epc_functions import _int, _signed_int

# 0xD0 System-interconnected type
def _0279D0(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x00: "System interconnected (reverse power flow acceptable)",
              0x01: "Independent",
              0x02: "System interconnected (reverse power flow not acceptable)",
              0x03: "Unknown"}
    return values.get(op_mode, "Invalid setting")

# 0xD1 Output power restraint status
def _0279D1(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "Ongoing restraint (output power control)",
              0x42: "Ongoing restraint (except output power control)",
              0x43: "Ongoing restraint (reason for restraint unknown)",
              0x44: "Not restraining",
              0x45: "Unknown"}
    return values.get(op_mode, "Invalid setting")

class HomeSolarPower(EchonetInstance):
    EPC_FUNCTIONS = {
        0xD0: _0279D0,
        0xD1: _0279D1,
        0xE0: _int,
        0xE1: _int,
        0xE2: _int,
        0xE3: _int,
        0xE4: _int,
        0xE5: _int,
        0xE6: _int,
        0xE7: _int,
        0xE8: _int,
        0xE9: _int
    }

    def __init__(self, host, api_connector, instance=0x1):
        self._eojgc = 0x02
        self._eojcc = 0x79
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    def getMeasuredInstantPower(self):
        return self.getMessage(0xE0)

    def getMeasuredCumulPower(self):
        return self.getMessage(0xE1)



from pychonet.EchonetInstance import EchonetInstance

"""Class for Electric Vehicle Charger Objects"""


class ElectricVehicleCharger(EchonetInstance):
    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x02  # Housing/facility-related device group
        self._eojcc = 0x7E  # Electric vehicle charger/discharger’
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )



# ------------------------------------------------------------------
# EPC set
# ---------------------------------------------------------------- */
EPC_CODE = {
    0x00: {  # Sensor-related device class group
        0x01: {  # Gas leak sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Gas leak occurrence status",
            0xBF: "Gas leak occurrence status resetting",
        },
        0x02: {  # Crime prevention sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Invasion occurrence status",
            0xBF: "Invasion occurrence status resetting",
        },
        0x03: {  # Emergency button class
            0x80: "Operation status",
            0xB1: "Emergency occurrence status",
            0xBF: "Emergency occurrence status resetting",
        },
        0x04: {  # First-aid sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "First-aid occurrence status",
            0xBF: "First-aid occurrence status resetting",
        },
        0x05: {  # Earthquake sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Earthquake occurrence status",
            0xBF: "Earthquake occurrence status resetting",
        },
        0x06: {  # Electric leak sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Electric leak occurrence status",
            0xBF: "Electric leak occurrence status resetting",
        },
        0x07: {  # Human detection sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Human detection status",
        },
        0x08: {  # Visitor sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Visitor detection status",
            0xBE: "Visitor detection holding time",
        },
        0x09: {  # Call sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Call status",
            0xBE: "Call holding time",
        },
        0x0A: {  # Condensation sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Condensation detection status",
        },
        0x0B: {  # Air pollution sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Air pollution detection status",
        },
        0x0C: {  # Oxygen sensor class
            0x80: "Operation status",
            0xE0: "Measured value of oxygen concentration",
        },
        0x0D: {  # Illuminance sensor class
            0x80: "Operation status",
            0xE0: "Measured illuminance value 1",
            0xE1: "Measured illuminance value 2",
        },
        0x0E: {  # Sound sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Sound detection status",
            0xBE: "Sound detection holding time",
        },
        0x0F: {  # Mailing sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Mailing detection status",
        },
        0x10: {  # Weight sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Weight detection status",
        },
        0x11: {  # Temperature sensor class
            0x80: "Operation status",
            0xE0: "Measured temperature value",
        },
        0x12: {  # Humidity sensor class
            0x80: "Operation status",
            0xE0: "Measured value of relative humidity",
        },
        0x13: {  # Rain sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Rain detection status",
        },
        0x14: {  # Water level sensor class
            0x80: "Operation status",
            0xB0: "Water level over detection threshold level",
            0xB1: "Water level over detection status",
            0xE0: "Measured value of water level",
        },
        0x15: {  # Bath water level sensor class
            0x80: "Operation status",
            0xB0: "Bath water level over detection threshold level",
            0xB1: "Bath water level over detection status",
            0xE0: "Measured value of bath water level",
        },
        0x16: {  # Bath heating status sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Bath heating detection status",
        },
        0x17: {  # Water leak sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Water leak detection status",
        },
        0x18: {  # Water overflow sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Water overflow detection status",
        },
        0x19: {  # Fire sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Fire occurrence detection status",
            0xBF: "Fire occurrence detection status resetting",
        },
        0x1A: {  # Cigarette smoke sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Smoke (cigarette) detection status",
        },
        0x1B: {  # CO2 sensor class
            0x80: "Operation status",
            0xE0: "Measured value of CO2 concentration",
        },
        0x1C: {  # Gas sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Gas detection status",
            0xE0: "Measured value of gas concentration",
        },
        0x1D: {  # VOC sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "VOC detection status",
            0xE0: "Measured value of VOC concentration",
        },
        0x1E: {  # Differential pressure sensor class
            0x80: "Operation status",
            0xE0: "Measured value of differential pressure",
        },
        0x1F: {  # Air speed sensor class
            0x80: "Operation status",
            0xE0: "Measured value of air speed",
            0xE1: "Air flow direction",
        },
        0x20: {  # Odor sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Odor detection status",
            0xE0: "Measured odor value",
        },
        0x21: {  # Flame sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Flame detection status",
            0xBF: "Flame detection status resetting",
        },
        0x22: {  # Electric energy sensor class
            0x80: "Operation status",
            0xE0: "Cumulative amounts of electric energy",
            0xE1: "Medium-capacity sensor instantaneous electric energy",
            0xE2: "Small-capacity sensor instantaneous electric energy",
            0xE3: "Large-capacity sensor instantaneous electric energy",
            0xE4: "Cumulative amounts of electric energy measurement log",
            0xE5: "Effective voltage value",
        },
        0x23: {  # Current value sensor class
            0x80: "Operation status",
            0xE0: "Measured current value 1",
            0xE1: "Rated voltage to be measured",
            0xE2: "Measured current  value 2",
        },
        0x25: {  # Water flow rate sensor class
            0x80: "Operation status",
            0xE0: "Cumulative flow rate",
            0xE2: "Flow rate",
        },
        0x26: {  # Micromotion sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Micromotion detection status",
            0xB2: "Detection counter",
            0xBC: "Sampling count",
            0xBD: "Sampling cycle",
        },
        0x27: {  # Passage sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xBE: "Passage detection hold time",
            0xE0: "Passage detection direction",
        },
        0x28: {  # Bed presence sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Bed presence detection status",
        },
        0x29: {  # Open/close sensor class
            0x80: "Operation status",
            0xE0: "Degree-of-opening detection status 1",
            0xB0: "Detection threshold level",
            0xB1: "Degree-of-opening detection status 2",
        },
        0x2A: {  # Activity amount sensor class
            0x80: "Operation status",
            0xE0: "Activity amount level 1",
            0xE1: "Maximum number of human body ID’s",
            0xE2: "Activity amount level 2",
            0xE3: "Human body existence information",
        },
        0x2B: {  # Human body location sensor
            0x80: "Operation status",
            0xE0: "Human body detection location 1",
            0xE1: "Maximum number of human body ID’s",
            0xE2: "Human body detection location 2",
            0xE3: "Human body existence information",
        },
        0x2C: {  # Snow sensor class
            0x80: "Operation status",
            0xB0: "Detection threshold level",
            0xB1: "Snow detection status",
        },
        0x2D: {  # Air pressure sensor class
            0x80: "Operation status",
            0xE0: "Air pressure measurement",
        },
    },
    0x01: {  # Air conditioner-related device class group
        0x30: {  # Home air conditioner class
            0x80: "Operation status",  # done
            0x8F: "Operation power-saving",
            0xB0: "Operation mode setting",  # done
            0xB1: "Automatic temperature control setting",
            0xB2: "Normal/high-speed/silent operation setting",
            0xB3: "Set temperature value",  # done
            0xB4: "Set value of relative humidity in dehumidifying mode",
            0xB5: "Set temperature value in cooling mode",
            0xB6: "Set temperature value in heating mode",
            0xB7: "Set temperature value in dehumidifying mode",
            0xB8: "Rated power consumption",
            0xB9: "Measured value of current consumption",
            0xBA: "Measured value of room relative humidity",
            0xBB: "Measured value of room temperature",  # done
            0xBC: "Set temperature value of user remote control",
            0xBD: "Measured cooled air temperature",
            0xBE: "Measured outdoor air temperature",  # done
            0xBF: "Relative temperature setting",
            0xA0: "Air flow rate setting",  # done
            0xA1: "Automatic control of air flow direction setting",  # done
            0xA3: "Automatic swing of air flow setting",  # done
            0xA4: "Air flow direction (vertical) setting",  # done
            0xA5: "Air flow direction (horizontal) setting",  # done
            0xAA: "Special state",
            0xAB: "Non-priority state",
            0xC0: "Ventilation function setting",
            0xC1: "Humidifier function setting",
            0xC2: "Ventilation air flow rate setting",
            0xC4: "Degree of humidification setting",
            0xC6: "Mounted air cleaning method",
            0xC7: "Air purifier function setting",
            0xC8: "Mounted air refresh method",
            0xC9: "Air refresher function setting",
            0xCA: "Mounted self-cleaning method",
            0xCB: "Self-cleaning function setting",
            0xCC: "Special function setting",
            0xCD: "Operation status of components",
            0xCE: "Thermostat setting override function",
            0xCF: "Air purification mode setting",
            0x90: "ON timer-based reservation setting",
            0x91: "ON timer setting (time)",
            0x92: "ON timer setting (relative time)",
            0x94: "OFF timer-based reservation setting",
            0x95: "OFF timer setting (time)",
            0x96: "OFF timer setting (relative time)",
        },
        0x33: {  # Ventilation fan class
            0x80: "Operation status",
            0xBF: "Ventilation auto setting",
            0xA0: "Set value of ventilation air flow rate",
        },
        0x34: {  # Air conditioner ventilation fan class
            0x80: "Operation status",
            0xB4: "Set value of room relative humidity",
            0xBF: "Ventilation auto setting",
            0xBA: "Measured value of room relative humidity",
            0xA0: "Set value of ventilation air flow rate",
            0xE0: "Heat exchanger operation setting",
            0xC0: "Measured value of CO2 concentration",
            0xC1: "Smoke (cigarette) detection status",
            0xC2: "Pollution detection status",
        },
        0x35: {  # Air cleaner class
            0x80: "Operation status",
            0xE1: "Filter change notice",
            0xA0: "Air flow rate setting",
            0xC1: "Smoke (cigarette) detection status",
            0xC2: "Optical catalyst operation setting",
            0xC0: "Air pollution detection status",
        },
        0x39: {  # Humidifier class
            0x80: "Operation status",
            0xC0: "Humidifying setting 1",
            0xC1: "Humidifying setting 2",
            0xB4: "Measured value of relative humidity",
            0x94: "Reservation set of OFF timer",
            0x96: "Relative time value set of OFF timer",
            0xC2: "Ion emission setting",
            0xC3: "Implemented ion emission method",
            0xC4: "Special operation mode setting",
            0xC5: "Water amount level",
        },
        0x42: {  # Electric heater class
            0x80: "Operation status",
            0xB1: "Automatic temperature control setting",
            0xB3: "Temperature setting",
            0xBB: "Measured room temperature",
            0xBC: "Remotely set temperature",
            0xA0: "Air flow rate setting",
            0x90: "ON timer-based reservation setting",
            0x91: "ON timer setting (time)",
            0x92: "ON timer setting (relative time)",
            0x94: "OFF timer-based reservation setting",
            0x95: "OFF timer setting (time)",
            0x96: "Set value of OFF timer relative time",
        },
        0x43: {  # Fan heater class
            0x80: "Operation status",
            0xB3: "Temperature setting value",
            0xBB: "Measured temperature",
            0xB1: "Automatic temperature control setting",
            0x90: "ON timer reservation setting",
            0x91: "ON timer setting value (time)",
            0x92: "ON timer setting value (relative time)",
            0x94: "OFF timer reservation setting",
            0x95: "OFF timer setting value (time)",
            0x96: "OFF timer value ( relative time)",
            0xC0: "Extensional operation setting",
            0xC1: "Extensional operation timer time setting value",
            0xC2: "Ion emission setting",
            0xC3: "Implemented ion emission method",
            0xC4: "Oil amount level",
        },
        0x45: {  # Package-type commercial air conditioner (indoor unit) class
            0x80: "Operation status",
            0xB0: "Operation mode setting",
            0xB3: "Temperature setting 1",
            0xB4: "“Relative humidity setting for ‘dehumidifica-tion’ mode” 1",
            0xB5: "“Temperature setting for ‘cooling’ mode” 1",
            0xB6: "“Temperature setting for ‘heating’ mode” 1",
            0xB7: "“Temperature setting for ‘dehumidifica-tion’ mode” 1",
            0xB8: "Rated power consumption of indoor unit",
            0xB9: "Measured electric current consumption of indoor unit",
            0xBA: "Measured indoor relative humidity 1",
            0xBB: "Measured indoor temperature 1",
            0xBF: "Relative temperature setting",
            0xA0: "Air flow rate setting",
            0xA4: "“Air flow direction (vertical)” setting",
            0xA5: "“Air flow direction (horizontal)” setting",
            0xAA: "“Special” state",
            0xAC: "Thermostat state",
            0xAE: "Current function (“automatic” operation mode)",
            0xC0: "Ventilation mode setting",
            0xC1: "Combined operation of indoor unit and total heat exchanger",
            0xC2: "Ventilation air flow rate setting",
            0xCD: "“Disabling of air conditioner” setting",
            0xCA: "Group information",
            0xCE: "Thermostat setting override function",
            0xCF: "Filter cleaning reminder lamp setting",
            0xDB: "Measured power consumption of indoor unit",
            0xDC: "Aperture of expansion valve",
            0xE3: "Temperature setting 2",
            0xE4: "“Relative humidity setting for ‘dehumidification’ mode” 2",
            0xE5: "“Temperature setting for ‘cooling’ mode” 2",
            0xE6: "“Temperature setting for ‘heating’ mode” 2",
            0xE7: "“Temperature setting for ‘dehumidification’ mode” 2",
            0xEA: "Measured indoor relative humidity 2",
            0xEB: "Measured indoor temperature 2",
            0x90: "“ON timer-based reservation” setting",
            0x91: "ON timer setting (time)",
            0x92: "ON timer setting (relative time)",
            0x94: "“OFF timer-based reservation” setting",
            0x95: "OFF timer setting (time)",
            0x96: "OFF timer setting (relative time)",
        },
        0x46: {  # Package-type commercial air conditioner (outdoor unit) class
            0x80: "Operation status",
            0xB0: "Operation mode setting",
            0xB8: "Rated power consumption of outdoor unit",
            0xB9: "Measured electric current consumption of outdoor unit",
            0xBE: "Measured outdoor air temperature 1",
            0xAA: "“Special” state",
            0xCA: "Group information",
            0xD0: "Operation status of compressor",
            0xD1: "Operation mode information",
            0xD2: "Fan rotation speed",
            0xDB: "Measured power consumption of outdoor unit",
            0xEE: "Measured outdoor air temperature 2",
        },
        0x55: {  # Electric storage heater class
            0x80: "Operation status",
            0xB3: "Temperature setting",
            0xB8: "Rated power consumption",
            0xBB: "Measured indoor temperature",
            0xBE: "Measured outdoor temperature",
            0xA0: "Air flow rate setting",
            0xA1: "Fan operation status",
            0xC0: "Heat storage operation status",
            0xC1: "Heat storage temperature setting",
            0xC2: "Measured stored heat temperature",
            0xC3: "Daytime heat storage setting",
            0xC4: "Daytime heat storage ability",
            0xC5: "Midnight power duration setting",
            0xC6: "Midnight power start time setting",
            0xC7: "Radiation method",
            0xC8: "Child lock setting",
            0xD0: "Fan timer 1 setting",
            0xD1: "Fan timer 1 ON time setting",
            0xD2: "Fan timer 1 OFF time setting",
            0xD3: "Fan timer 2 setting",
            0xD4: "Fan timer 2 ON time setting",
            0xD5: "Fan timer 2 OFF time setting",
        },
    },
    0x02: {  # Housing/facility-related device class group
        0x60: {  # Electrically operated blind/shade class
            0x80: "Operation status",
            0x89: "Fault description (Recoverable faults)",
            0x90: "Timer operation setting",
            0xC2: "Wind detection status",
            0xC3: "Sunlight detection status",
            0xD0: "Opening (extension) speed setting",
            0xD1: "Closing (retraction) speed setting",
            0xD2: "Operation time",
            0xD4: "Automatic operation setting",
            0xE0: "Open/close (extension/retraction) setting",
            0xE1: "Degree-of-opening level",
            0xE2: "Shade angle setting ",
            0xE3: "Open/close (extension/retraction) speed",
            0xE5: "Electric lock setting",
            0xE8: "Remote operation setting status",
            0xE9: "Selective opening (extension) operation setting",
            0xEA: "Open/closed (extended/retracted) status",
            0xEE: "One-time opening (extension) speed setting",
            0xEF: "One-time closing (retraction) speed setting",
        },
        0x61: {  # Electrically operated shutter class
            0x80: "Operation status",
            0x89: "Fault description (Recoverable faults)",
            0x90: "Timer operation setting",
            0xD0: "Opening speed setting",
            0xD1: "Closing speed setting",
            0xD2: "Operation time",
            0xE0: "Open/close operation setting",
            0xE1: "Degree-of-opening setting",
            0xE2: "Blind angle setting",
            0xE3: "Open/close speed setting",
            0xE5: "Electric lock setting",
            0xE8: "Remote operation setting status",
            0xE9: "Selective degree-of-opening setting",
            0xEA: "Open/closed status",
            0xED: "Slit degree-of-opening setting",
            0xEE: "One-time opening speed setting",
            0xEF: "One-time closing speed setting",
        },
        0x63: {  # Electrically operated rain sliding door/shutter class
            0x80: "Operation status",
            0x89: "Fault description (Recoverable faults)",
            0x90: "Timer operation setting",
            0xD0: "Opening speed setting",
            0xD1: "Closing speed setting",
            0xD2: "Operation time",
            0xE0: "Open/close operation setting",
            0xE1: "Degree-of-opening setting",
            0xE2: "Blind angle setting",
            0xE3: "Opening/closing",
            0xE5: "Electric lock setting",
            0xE8: "Remote operation setting status",
            0xE9: "Selective degree-of-opening setting",
            0xEA: "Open/closed status",
            0xED: "Slit degree-of-opening",
            0xEE: "One-time opening speed setting",
            0xEF: "One-time closing speed setting",
        },
        0x64: {  # Electrically operated gate class
            0x80: "Operation status",
            0x89: "Fault description (Recoverable faults)",
            0xD0: "Opening speed setting",
            0xD1: "Closing speed setting",
            0xD2: "Operation time setting value",
            0xE0: "Opening/closing operation setting",
            0xE1: "Degree-of-opening setting",
            0xE3: "Opening/closing speed setting",
            0xE5: "Electric lock setting",
            0xE8: "Remote operation",
            0xE9: "Selective degree-of-opening setting",
            0xEA: "Open/closed status",
            0xEE: "One-time opening speed setting",
            0xEF: "One-time closing speed setting",
        },
        0x65: {  # Electrically operated window class
            0x80: "Operation status",
            0x89: "Fault description (Recoverable faults)",
            0x90: "Timer operation setting",
            0xC0: "Registered temperature detection status",
            0xC1: "Rain detection status",
            0xD0: "Opening speed setting",
            0xD1: "Closing speed setting",
            0xD2: "Operation time setting value",
            0xD4: "Automatic operation setting",
            0xE0: "Opening/closing operation setting",
            0xE1: "Degree-of-opening setting",
            0xE3: "Opening/closing speed setting",
            0xE5: "Electric lock setting",
            0xE8: "Remote operation setting status",
            0xE9: "Selective degree-of-opening setting",
            0xEA: "Open/closed status",
            0xEE: "One-time opening speed setting",
            0xEF: "One-time closing speed setting",
        },
        0x66: {  # Automatically operated entrance door/sliding door class
            0x80: "Operation status",
            0x89: "Fault description (Recoverable faults)",
            0xD0: "Opening speed setting",
            0xD1: "Closing speed setting",
            0xD2: "Operation time setting value",
            0xD5: "Opening time setting",
            0xE0: "Opening/closing operation setting",
            0xE1: "Degree-of-opening setting",
            0xE3: "Opening/closing speed setting",
            0xE8: "Remote operation setting status",
            0xE9: "Selective degree-of-opening setting",
            0xEA: "Open/closed status",
            0xEE: "One-time opening speed setting",
            0xEF: "One-time closing speed setting",
        },
        0x67: {  # Garden sprinkler class
            0x80: "Operation status",
            0xE0: "Sprinkle valve open/close setting",
            0xE1: "Sprinkle interval setting",
            0xE2: "Number of sprinkles setting",
            0xE3: "Sprinkle time setting 1",
            0xE4: "Sprinkle time setting 2",
            0xE5: "Sprinkle duration setting",
        },
        0x6B: {  # Electric water heater class
            0x80: "Operation status",
            0xB0: "Automatic water heating setting",
            0xB1: "Automatic water temperature control setting",
            0xB2: "Water heater status",
            0xB3: "Water heating temperature setting",
            0xB4: "Manual water heating stop days setting",
            0xB5: "Relative time setting value for manual water heating OFF",
            0xB6: "Tank operation mode setting",
            0xC0: "Daytime reheating permission setting",
            0xC1: "Measured temperature of water in water heater",
            0xC2: "Alarm status",
            0xC3: "Hot water supply status",
            0xC4: "Relative time setting for keeping bath temperature",
            0xD1: "Temperature of supplied water setting",
            0xD3: "Bath water temperature setting",
            0xE0: "Bath water volume setting",
            0xE1: "Measured amount of water remaining in tank",
            0xE2: "Tank capacity",
            0xE3: "Automatic bath water heating mode setting",
            0xE9: "Bathroom priority setting",
            0xEA: "Bath operation status monitor",
            0xE4: "Manual bath reheating operation setting",
            0xE5: "Manual bath hot water addition function setting",
            0xE6: "Manual slight bath water temperature lowering function setting",
            0xE7: "Bath water volume setting 1",
            0xE8: "Bath water volume setting 2",
            0xEE: "Bath water volume setting 3",
            0xD4: "Bath water volume setting 4",
            0xD5: "Bath water volume setting 4-Maximum settable level",
            0x90: "ON timer reservation setting",
            0x91: "ON timer setting",
            0xD6: "Volume setting",
            0xD7: "Mute setting",
            0xD8: "Remaining hot water volume",
            0xDB: "Rated power consumption of H/P unit in wintertime",
            0xDC: "Rated power consumption of H/P unit in in-between seasons",
            0xDD: "Rated power consumption of H/P unit in summertime",
        },
        0x6E: {  # Bidet-equipped toilet (with electrically warmed seat) class
            0x80: "Operation status",
            0xE0: "Temperature level of toilet seat",
            0xE1: "Heater setting of toilet seat",
            0xE2: "Temporal halt setting of toilet seat",
            0xE3: "Temporal halt start time of toilet seat",
            0xE4: "Temporal halt time duration of toilet seat",
            0xE5: "Temperature level setting of room heating",
            0xE6: "Room heating setting",
            0xE7: "Room heating status",
            0xE8: "Start time of room heating",
            0xE9: "Duration time of room heating",
            0xEA: "Special operation mode setting",
            0xEB: "Human detection status",
            0xEC: "Seating detection status",
        },
        0x6F: {  # Electric lock class
            0x80: "Operation status",
            0xE0: "Lock setting 1",
            0xE1: "Lock setting 2",
            0xE2: "Lock status of door guard",
            0xE3: "Door open/close status",
            0xE4: "Occupant/non-occupant status",
            0xE5: "Alarm status",
            0xE6: "Auto lock mode setting",
            0xE7: "Battery level",
        },
        0x72: {  # Hot water generator class
            0x80: "Operation status",
            0xD0: "Hot water heating status",
            0xD1: "Set value of hot water temperature",
            0xD2: "Hot water warmer setting",
            0xDA: "“Duration of automatic operation” setting",
            0xDB: "Remaining automatic operation time",
            0xE1: "Set value of bath temperature",
            0xE2: "Bath water heater status",
            0xE3: "Bath auto mode setting",
            0xE4: "Bath additional boil-up operation setting",
            0xE5: "Bath hot water adding operation setting",
            0xE6: "Bath water temperature lowering operation setting",
            0xE7: "Bath hot water volume setting 1",
            0xE8: "Bath hot water volume setting 2",
            0xEE: "Bath hot water volume setting 3",
            0xD4: "Bath hot water volume setting 4",
            0xD5: "Bath hot water volume setting 4 - Maximum settable level",
            0xE9: "Bathroom priority setting",
            0xEA: "Shower hot water supply status",
            0xEB: "Kitchen hot water supply status",
            0xEC: "Hot water warmer ON timer reservation setting",
            0xED: "Set value of hot water warmer ON timer time",
            0xEF: "Bath operation status monitor",
            0x90: "ON timer reservation setting",
            0x91: "Set value of ON timer time",
            0x92: "Set value of ON timer relative time",
            0xD6: "Volume setting",
            0xD7: "Mute setting",
        },
        0x73: {  # Bathroom dryer class
            0x80: "Operation status",
            0xB0: "Operation setting",
            0xB1: "Ventilation operation setting",
            0xB2: "Bathroom pre-warmer operation setting",
            0xB3: "Bathroom heater operation setting",
            0xB4: "Bathroom dryer operation setting",
            0xB5: "Cool air circulator operation setting",
            0xBA: "Measured relative bathroom humidity",
            0xBB: "Measured bathroom temperature",
            0xC2: "Ventilation air flow rate setting",
            0xCF: "Filter cleaning reminder sign setting",
            0xE0: "Human body detection status",
            0x90: "“ON timer-based reservation” setting 1",
            0xE1: "“ON timer-based reservation” setting 2",
            0x91: "ON timer setting (time)",
            0x92: "ON timer setting (relative time)",
            0x94: "“OFF timer-based reservation” setting",
            0x95: "OFF timer setting (time)",
            0x96: "OFF timer setting (relative time)",
        },
        0x79: {  # Home solar power generation class
            0x80: "Operation status",
            0xD0: "System-interconnected type",
            0xD1: "Output power restraint status",
            0xE0: "Measured instantaneous amount of electricity generated",
            0xE1: "Measured cumulative amount of electric energy generated",
            0xE2: "Resetting cumulative amount of electric energy generated",
            0xE3: "Measured cumulative amount of electric energy sold",
            0xE4: "Resetting cumulative amount of electric energy sold",
            0xE5: "Power generation output limit setting 1",
            0xE6: "Power generation output limit setting 2",
            0xE7: "Limit setting for the amount of electricity sold",
            0xE8: "Rated power generation output (System-interconnected)",
            0xE9: "Rated power generation output (Independent)",
        },
        0x7A: {  # Cold/hot water heat source equipment class
            0x80: "Operation status",
            0xE0: "Operation mode setting",
            0xE1: "Water temperature setting 1",
            0xE2: "Water temperature setting 2",
            0xE3: "Measured temperature of outward water (Exit water Temperature)",
            0xE4: "Measured temperature of inward water (Entrance water temperature)",
            0xE5: "Special operation setting",
            0xE6: "Daily timer setting",
            0xE7: "Daily timer setting 1",
            0xE8: "Daily timer setting 2",
            0xE9: "ON timer reservation setting",
            0x91: "ON timer setting",
            0x92: "Relative ON timer setting",
            0x94: "OFF timer reservation setting",
            0x95: "Time set by OFF timer",
            0x96: "Relative OFF timer setting",
        },
        0x7B: {  # Floor heater class
            0x80: "Operation status",
            0xE0: "Temperature setting 1",
            0xE1: "Temperature setting 2",
            0xE2: "Measured room temperature",
            0xE3: "Measured floor temperature",
            0xE4: "Zone change setting",
            0xE5: "Special operation setting",
            0xE6: "Daily timer setting",
            0xE7: "Daily timer setting 1",
            0xE8: "Daily timer setting 2",
            0x90: "ON timer reservation setting",
            0x91: "Time set by ON timer",
            0x92: "Relative ON timer setting",
            0x94: "OFF timer reservation setting",
            0x95: "Time set by OFF timer",
            0x96: "Relative OFF timer setting",
        },
        0x7C: {  # Fuel cell class
            0x80: "Operation status",
            0xC1: "Measured temperature of water in water heater",
            0xC2: "Rated power generation output",
            0xC3: "Heating value of hot water storage tank",
            0xC4: "Measured instantaneous power generation output",
            0xC5: "Measured cumulative power generation output",
            0xC6: "Cumulative power generation output reset setting",
            0xC7: "Measured instantaneous gas consumption",
            0xC8: "Measured cumulative gas consumption",
            0xC9: "Cumulative gas consumption reset setting",
            0xCA: "Power generation setting",
            0xCB: "Power generation status",
            0xCC: "Measured in-house instantaneous power consumption",
            0xCD: "Measured in-house cumulative power consumption",
            0xCE: "In-house cumulative power consumption reset",
            0xD0: "System interconnected type",
            0xE1: "Measured remaining hot water amount",
            0xE2: "Tank capacity",
        },
        0x7D: {  # Storage battery class
            0x80: "Operation status",
            0xC8: "Minimum/maxim um charging electric energy",
            0xC9: "Minimum/maxim um discharging electric energy",
            0xCA: "Minimum/maxim um charging current",
            0xCB: "Minimum/maxim um discharging current",
            0xCF: "Working operation status",
            0xD0: "Rated electric energy",
            0xD1: "Rated capacity",
            0xD2: "Rated voltage",
            0xD3: "Measured instantaneous charging/discharging electric energy",
            0xD4: "Measured instantaneous charging/discharging current",
            0xD5: "Measured instantaneous charging/discharging voltage",
            0xD6: "Measured cumulative discharging electric energy",
            0xD7: "“Measured cumulative discharging electric energy” reset setting",
            0xD8: "Measured cumulative charging electric energy",
            0xD9: "“Measured cumulative charging electric energy” reset setting",
            0xDA: "Operation mode setting",
            0xDB: "System-interconne cted type",
            0xDC: "Minimum/maxim um charging power (Independent)",
            0xDD: "Minimum/maxim um discharging power (Independent)",
            0xDE: "Minimum/maxim um charging current (Independent)",
            0xDF: "Minimum/maxim um discharging current (Independent)",
            0xE0: "Charging/discharging amount setting 1",
            0xE1: "Charging/discharging amount setting 2",
            0xE2: "Remaining stored electricity 1",
            0xE3: "Remaining stored electricity 2",
            0xE4: "Remaining stored electricity 3",
            0xE5: "Battery state of health",
            0xE6: "Battery type",
            0xE7: "Charging amount setting 1",
            0xE8: "Discharging amount setting 1",
            0xE9: "Charging amount setting 2",
            0xEA: "Discharging amount setting 2",
            0xEB: "Charging electric energy setting",
            0xEC: "Discharging electric energy setting",
            0xED: "Charging current setting",
            0xEE: "Discharging current setting",
            0xEF: "Rated voltage (Independent)",
        },
        0x7E: {  # Electric vehicle charger/discharger class
            0x80: "Operation status",
            0xC0: "Dischargeable capacity of vehicle mounted battery 1",
            0xC1: "Dischargeable capacity of vehicle mounted battery 2",
            0xC2: "Remaining dischargeable capacity of vehicle mounted battery 1",
            0xC3: "Remaining dischargeable capacity of vehicle mounted battery 2",
            0xC4: "Remaining dischargeable capacity of vehicle mounted battery 3",
            0xC5: "Rated charge capacity",
            0xC6: "Rated discharge capacity",
            0xC7: "Vehicle connection and chargeable/dischargeable status",
            0xC8: "Minimum/maximum charging electric energy",
            0xC9: "Minimum/maximum discharging electric energy",
            0xCA: "Minimum/maximum charging current",
            0xCB: "Minimum/maximum discharging current",
            0xD0: "Used capacity of vehicle mounted battery 1",
            0xD1: "Used capacity of vehicle mounted battery 2",
            0xD2: "Rated voltage",
            0xD3: "Measured instantaneous charging/discharging electric energy",
            0xD4: "Measured instantaneous charging/discharging current",
            0xD5: "Measured instantaneous charging/discharging voltage",
            0xD6: "Measured cumulative amount of discharging electric energy",
            0xD7: "Cumulative amount of discharging electric energy reset setting",
            0xD8: "Measured cumulative amount of charging electric energy",
            0xD9: "Cumulative amount of charging electric energy reset setting",
            0xDA: "Operation mode setting",
            0xDB: "System-interconnecte d type",
            0xE2: "Remaining stored electricity of vehicle mounted battery 1",
            0xE3: "Remaining stored electricity of vehicle mounted battery 2",
            0xE4: "Remaining stored electricity of vehicle mounted battery 3",
            0xE7: "Charging amount setting 1",
            0xE9: "Charging amount setting 2",
            0xEB: "Charging electric energy setting",
            0xEC: "Discharging electric energy setting",
            0xED: "Charging current setting",
            0xEE: "Discharging current setting",
            0xEF: "Rated voltage (Independent)",
        },
        0x7F: {  # Engine cogeneration class
            0x80: "Operation status",
            0xC1: "Measured temperature of water in water heater",
            0xC2: "Rated power generation output",
            0xC3: "Heating value of hot water storage tank",
            0xC4: "Measured instantaneous power generation output",
            0xC5: "Measured cumulative power generation output",
            0xC6: "Cumulative power generation output reset setting",
            0xC7: "Measured instantaneous gas consumption",
            0xC8: "Measured cumulative gas consumption",
            0xC9: "Cumulative gas consumption reset setting",
            0xCA: "Power generation setting",
            0xCB: "Power generation status",
            0xCC: "Measured in-house instantaneous power consumption",
            0xCD: "Measured in-house cumulative power consumption",
            0xCE: "In-house cumulative power consumption reset",
            0xD0: "System interconnected type",
            0xE1: "Measured remaining hot water amount",
            0xE2: "Tank capacity",
        },
        0x80: {  # Electric energy meter class
            0x80: "Operation status",
            0xE0: "Cumulative amounts of electric energy measurement value",
            0xE2: "Cumulative amounts of electric energy unit",
            0xE3: "Cumulative amounts of electric energy measurement log 1",
            0xE4: "Cumulative amounts of electric energy measurement log 2",
        },
        0x81: {  # Water flow meter class
            0x80: "Operation status",
            0xD0: "Flowing water classification",
            0xD1: "Owner classification",
            0xE0: "Measured cumulative amount of flowing water",
            0xE1: "Unit for measured cumulative amounts of flowing water",
            0xE2: "Historical data of measured cumulative amounts of flowing water",
            0xE3: "Detection of abnormal value in metering data",
            0xE4: "Security data information",
            0xE5: "ID number setting",
            0xE6: "Verification expiration information",
        },
        0x82: {  # Gas meter class
            0x80: "Operation status",
            0xE0: "Cumulative amount of gas consumption measurement value",
            0xE2: "Cumulative amount of gas consumption measurement log",
        },
        0x83: {  # LP gas meter class
            0x80: "Operation status",
            0xE0: "Cumulative amount of gas consumption of metering data 1",
            0xE1: "Cumulative amount of gas consumption of metering data 2",
            0xE2: "Error detection status of metering data",
            0xE3: "Security data 1",
            0xE4: "Security data 2",
            0xE5: "Center valve shut-off status",
            0xE6: "Center valve shut-off recovery permission setting status",
            0xE7: "Emergency valve shut-off status",
            0xE8: "Shut-off valve open/close status",
            0xE9: "Residual volume control warning",
            0xEA: "Set value of residual volume control warning level 1",
            0xEB: "Set value of residual volume control warning level 2",
            0xEC: "Set value of residual volume control warning level 3",
            0xED: "Slight leak timer value (gas flow rate continuation)",
            0xEE: "Slight leak timer value (without pressure increase)",
            0xEF: "Shut-off reason log",
            0xD0: "Maximum value of supply pressure data",
            0xD1: "Minimum value of supply pressure data",
            0xD2: "Current value of supply pressure data",
            0xD3: "Maximum value of block pressure data",
            0xD4: "Minimum value of block pressure data",
            0xD5: "Current value of block pressure data",
            0xD6: "Number of block pressure/supply pressure error days: times",
            0xD7: "Test call setting",
        },
        0x87: {  # Distribution panel metering class
            0x80: "Operation status",
            0xC0: "Measured cumulative amount of electric energy (normal direction)",
            0xC1: "Measured cumulative amount of electric energy (reverse direction)",
            0xC2: "Unit for cumulative amounts of electric energy",
            0xC3: "Historical data of measured cumulative amounts of electric energy (normal direction)",
            0xC4: "Historical data of measured cumulative amounts of electric energy (reverse direction)",
            0xC5: "Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved",
            0xC6: "Measured instantaneous amount of electric energy",
            0xC7: "Measured instantaneous currents",
            0xC8: "Measured instantaneous voltages",
            0xD0: "Measurement channel 1",
            0xD1: "Measurement channel 2",
            0xD2: "Measurement channel 3",
            0xD3: "Measurement channel 4",
            0xD4: "Measurement channel 5",
            0xD5: "Measurement channel 6",
            0xD6: "Measurement channel 7",
            0xD7: "Measurement channel 8",
            0xD8: "Measurement channel 9",
            0xD9: "Measurement channel 10",
            0xDA: "Measurement channel 11",
            0xDB: "Measurement channel 12",
            0xDC: "Measurement channel 13",
            0xDD: "Measurement channel 14",
            0xDE: "Measurement channel 15",
            0xDF: "Measurement channel 16",
            0xE0: "Measurement channel 17",
            0xE1: "Measurement channel 18",
            0xE2: "Measurement channel 19",
            0xE3: "Measurement channel 20",
            0xE4: "Measurement channel 21",
            0xE5: "Measurement channel 22",
            0xE6: "Measurement channel 23",
            0xE7: "Measurement channel 24",
            0xE8: "Measurement channel 25",
            0xE9: "Measurement channel 26",
            0xEA: "Measurement channel 27",
            0xEB: "Measurement channel 28",
            0xEC: "Measurement channel 29",
            0xED: "Measurement channel 30",
            0xEE: "Measurement channel 31",
            0xEF: "Measurement channel 32",
            0xB0: "Master rated capacity",
            0xB1: "Number of measurement channels (simplex)",
            0xB2: "Channel range specification for cumulative amount of electric power consumption measurement (simplex)",
            0xB3: "Measured cumulative amount of electric power consumption list (simplex)",
            0xB4: "Channel range specification for instantaneous current measurement (simplex)",
            0xB5: "Measured instantaneous current list (simplex)",
            0xB6: "Channel range specification for instantaneous power consumption measurement (simplex)",
            0xB7: "Measured instantaneous power consumption list (simplex)",
            0xB8: "Number of measurement channels (duplex)",
            0xB9: "Channel range specification for cumulative amount of electric power consumption measurement (duplex)",
            0xBA: "Measured cumulative amount of electric power consumption list (duplex)",
            0xBB: "Channel range specification for instantaneous current measurement (duplex)",
            0xBC: "Measured instantaneous current list (duplex)",
            0xBD: "Channel range specification for instantaneous power consumption measurement (duplex)",
            0xBE: "Measured instantaneous power consumption list (duplex)",
        },
        0x88: {  # Low voltage smart electric energy meter class
            0x80: "Operation status",
            0xD3: "Coefficient",
            0xD7: "Number of effective digits for cumulative amounts of electric energy",
            0xE0: "Measured cumulative amount of electric energy (normal direction)",
            0xE1: "Unit for cumulative amounts of electric energy (normal and reverse directions)",
            0xE2: "Historical data of measured cumulative amounts of electric energy 1 (normal direction)",
            0xE3: "Measured cumulative amounts of electric energy (reverse direction)",
            0xE4: "Historical data of measured cumulative amounts of electric energy 1 (reverse direction)",
            0xE5: "Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved 1",
            0xE7: "Measured instantaneous electric energy",
            0xE8: "Measured instantaneous currents",
            0xEA: "Cumulative amounts of electric energy measured at fixed time (normal direction)",
            0xEB: "Cumulative amounts of electric energy measured at fixed time (reverse direction)",
            0xEC: "Historical data of measured cumulative amounts of electric energy 2 (normal and reverse directions)",
            0xED: "Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved 2",
        },
        0x89: {  # Smart gas meter class
            0x80: "Operation status",
            0xE0: "Gas meter classification setting",
            0xE1: "Owner classification setting",
            0xE2: "Measured cumulative gas consumption",
            0xE3: "Unit for measured cumulative gas consumption",
            0xE4: "Historical data of measured cumulative gas consumption",
            0xE5: "Day setting for which the historical data of measured cumulative gas consumption is to be retrieved",
            0xE6: "Detection of abnormal value in metering data",
            0xE7: "Security data information",
            0xE8: "Valve closure by the Center",
            0xE9: "Permission from the Center to reopen the valve closed by the Center",
            0xEA: "Emergency closure of shutoff valve",
            0xEB: "Shutoff valve status",
            0xEC: "Historical data of shutoff reasons",
            0xED: "ID number setting",
            0xEE: "Verification expiration setting",
            0xD0: "Measured cumulative gas consumption information with date and time",
            0xD1: "Historical information of cumulative gas consumption",
        },
        0x8A: {  # High voltage smart electric energy meter class
            0x80: "Operation status",
            0xD3: "Coefficient",
            0xD4: "Multiplying factor for coefficient",
            0xE0: "Fixed date",
            0xE1: "Day for which the historical data of measured cumulative amounts of electric energy is to be retrieved",
            0xE2: "Measured cumulative amount of active electric energy",
            0xE3: "Cumulative amounts of active electric energy at fixed time",
            0xE4: "Measurement data of cumulative amount of active electric energy for power factor measurement",
            0xE5: "Number of effective digits for cumulative amount of active electric energy",
            0xE6: "Unit for cumulative amounts of active electric energy",
            0xE7: "Historical data of measured cumulative amount of active electric energy",
            0xC1: "Monthly maximum electric power demand",
            0xC2: "Cumulative maximum electric power demand",
            0xC3: "Electric power demand at fixed time (30-minute average electric power)",
            0xC4: "Number of effective digits of electric power demand",
            0xC5: "Unit of electric power demand",
            0xC6: "Historical data of measured electric power demand",
            0xC7: "Unit of cumulative maximum electric power demand",
            0xCA: "Measurement data of reactive electric power consumption (lag) for power factor measurement",
            0xCB: "Measurement data of cumulative amount of reactive electric power consumption (lag) at fixed time for power factor measurement",
            0xCC: "Number of effective digits for measurement data of cumulative amount of reactive electric power consumption (lag) for power factor measurement",
            0xCD: "Unit of measurement data of cumulative amount of reactive electric power consumption (lag)",
            0xCE: "Historical data of measurement data of cumulative amount of reactive electric power consumption(lag) for power factor measurement",
        },
        0x90: {  # General lighting class class
            0x80: "Operation status",
            0xB0: "Illuminance level",
            0xB1: "Light color setting",
            0xB2: "Illuminance level step setting",
            0xB3: "Light color step setting",
            0xB4: "Maximum specifiable values",
            0xB5: "Maximum value of settable level for night lighting",
            0xB6: "Lighting mode setting",
            0xB7: "Illuminance level setting for main lighting",
            0xB8: "Illuminance level step setting for main lighting",
            0xB9: "Illuminance level setting for night lighting",
            0xBA: "Illuminance level step setting for night lighting",
            0xBB: "Light color setting for main lighting",
            0xBC: "Light color level step setting for main lighting",
            0xBD: "Light color setting for night lighting",
            0xBE: "Light color level step setting for night lighting",
            0xBF: "Lighting mode status in auto mode",
            0xC0: "RGB setting for color lighting",
            0x90: "ON timer reservation setting",
            0x91: "ON timer setting",
            0x94: "OFF timer reservation setting",
            0x95: "OFF timer setting",
        },
        0x91: {  # Single function lighting class
            0x80: "Operation status",
            0xB0: "Illuminance level setting",
        },
        0xA0: {  # Buzzer class
            0x80: "Operation status",
            0xB1: "Sound generation setting",
            0xE0: "Buzzer sound type",
        },
        0xA2: {  # Household small wind turbine power generation
            0x80: "Operation status",
            0xD0: "System interconnection status",
            0xE0: "Measured instantaneous amount of electricity",
            0xE1: "Measured cumulative amount of electricity",
            0xE2: "Resetting cumulative amount of electricity",
            0xE3: "Measured cumulative amount of electricity sold",
            0xE4: "Resetting cumulative amount of electricity sold",
            0xE5: "Power generation output limit setting 1",
            0xE6: "Power generation output limit setting 2",
            0xE7: "Limit setting for the amount of electricity sold",
            0xE8: "Rated power",
            0xEA: "Measured wind speed",
            0xEB: "Rated wind speed",
            0xEC: "Cut-in wind speed",
            0xED: "Cut-out wind speed",
            0xEE: "Extreme wind speed",
            0xEF: "Braking status",
        },
        0xA3: {  # Light Controller
            0x80: "Operation status",
            0xB0: "Illuminance level setting",
            0xC0: "Scene control setting",
            0xC1: "Number that can assign scene control setting",
        },
    },
    0x03: {  # Cooking/housework-related device class group
        0xB2: {  # Electric hot water pot (Electric thermos) class
            0x80: "Operation status",
            0xB0: "Cover open/close status",
            0xB1: "No-water warning",
            0xB2: "Boil-up setting",
            0xE0: "Boil-up/warmer mode setting",
            0xE1: "Set value of warmer temperature",
            0xE2: "Hot water discharge status",
            0xE3: "Lock status",
        },
        0xB7: {  # Refrigerator class
            0x80: "Operation status",
            0xB0: "Door open/close status",
            0xB1: "Door open warning",
            0xB2: "Refrigerator compartment door status",
            0xB3: "Freezer compartment door status",
            0xB4: "Ice compartment door status",
            0xB5: "Vegetable compartment door status",
            0xB6: "Multi-refrigera-ting mode compartment door",
            0xE0: "Maximum allowable temperature setting level",
            0xE2: "Refrigerator compartment temperature setting",
            0xE3: "Freezer compartment temperature setting",
            0xE4: "Ice temperature setting",
            0xE5: "Vegetable compartment temperature setting",
            0xE6: "Multi-refrigera-ting mode compartment temperature setting",
            0xE9: "Refrigerator compartment temperature level setting",
            0xEA: "Freezer compartment temperature level setting",
            0xEB: "ice compartment temperature level setting",
            0xEC: "Vegetable compartment temperature level setting",
            0xED: "Multi-refrigera-ting mode compartment temperature level setting",
            0xD1: "Measured refrigerator compartment temperature",
            0xD2: "Measured freezer compartment temperature",
            0xD3: "Measured subzero-fresh compartment temperature",
            0xD4: "Measured vegetable compartment temperature",
            0xD5: "Measured multi-refrigeratin g mode compartment temperature",
            0xD8: "Compressor rotation speed",
            0xDA: "Measured electric current consumption",
            0xDC: "Rated power consumption",
            0xA0: "Quick freeze function setting",
            0xA1: "Quick refrigeration function setting",
            0xA4: "Icemaker setting",
            0xA5: "Icemaker operation status",
            0xA6: "Icemaker tank status",
            0xA8: "Refrigerator compartment humidification function setting",
            0xA9: "Vegetable compartment humidification function setting",
            0xAD: "Deodorization function setting",
        },
        0xB8: {  # Combination microwave oven (Electronic oven) class
            0x80: "Operation status",
            0xB0: "Door open/close status",
            0xB1: "Heating status",
            0xB2: "Heating setting",
            0xE0: "Heating mode setting",
            0xE1: "Automatic heating setting",
            0xE2: "Automatic heating level setting",
            0xD0: "Automatic heating menu setting",
            0xD1: "Oven mode setting",
            0xD5: "Oven preheating setting",
            0xD6: "Fermenting mode setting",
            0xE3: "Chamber temperature setting",
            0xE4: "Food temperature setting",
            0xE5: "Heating time setting",
            0xE6: "Remaining heating time setting",
            0xE7: "Microwave heating power setting",
            0xE8: "Prompt message setting",
            0xE9: "Accessories to combination microwave oven setting",
            0xEA: "Display character string setting",
            0xEB: "Two-stage microwave heating setting (duration)",
            0xEC: "Two-stage microwave heating setting (heating power)",
        },
        0xB9: {  # Cooking heater class
            0x80: "Operation status",
            0xB1: "Heating status",
            0xB2: "Heating setting",
            0xB3: "All stop setting",
            0xE7: "Heating power setting",
            0xE3: "Heating temperature setting",
            0xE0: "“Heating modes of stoves” setting",
            0x96: "Relative time settings of OFF timers",
            0xA1: "Child lock setting",
            0xA2: "Radiant heater lock setting",
        },
        0xBB: {  # Raice cooker class
            0x80: "Operation status",
            0xB0: "Cover open/close status",
            0xB1: "Rice cooking status",
            0xB2: "Rice cooking control setting",
            0xE1: "Warmer setting",
            0xE5: "Inner pot removal status",
            0xE6: "Cover removal status",
            0x90: "Rice cooking reservation setting",
            0x91: "Set value of rice cooking reservation setting time",
            0x92: "Set value of rice cooking reservation setting relative time",
        },
        0xC5: {  # Washing machine class
            0x80: "Operation status",
            0xB0: "Door/cover open/close status",
            0xB2: "Washing machine setting",
            0xE1: "Current stage of washing cycle",
            0xE6: "Time remaining to complete washing cycle",
            0x90: "ON timer reservation setting",
            0x91: "ON timer setting",
            0x92: "Relative time-based ON timer setting",
        },
        0xCE: {  # Commercial show case class
            0x80: "Operation status",
            0xB0: "Operating mode",
            0xBD: "Discharge temperature measurement",
            0xE0: "Internal lighting operation status",
            0xE1: "External lighting operation status",
            0xE2: "Compressor operation status",
            0xE3: "Internal temperature measurement",
            0xE4: "Freezing capability value",
            0xE5: "Defrosting heater power consumption",
            0xE6: "Fan motor power consumption",
            0xCA: "Group information",
            0xE7: "Heater mode",
        },
        0xC6: {  # Clothes dryer class
            0x80: "Operation status",
            0xB0: "Door/cover open/close status",
            0xB2: "Drying setting",
            0xE1: "Drying status",
            0xE6: "Remaining drying time",
            0x90: "ON timer reservation setting",
            0x91: "ON timer setting",
            0x92: "Relative time-based ON timer setting",
        },
        0xD3: {  # Washer and dryer class
            0x80: "Operation status",
            0xB0: "Door/cover open/close status",
            0xB2: "Washer and dryer setting",
            0xD0: "Washer and dryer cycle setting 1",
            0xD1: "Washer and dryer cycle setting 2",
            0xD2: "Drying cycle setting",
            0xD3: "Washer and dryer cycle option list 1",
            0xD4: "Washer and dryer cycle option list 2",
            0xD5: "Washer and dryer cycle option list 3",
            0xD6: "Water flow rate setting",
            0xD7: "“Rotation speed for spin drying” setting",
            0xD8: "“Degree of drying” setting",
            0xDB: "Remaining washing time",
            0xDC: "Remaining drying time",
            0xDF: "Elapsed time on the ON timer",
            0xE1: "Presoaking time setting",
            0xE2: "Current stage of washer and dryer cycle",
            0xE3: "Water volume setting 1",
            0xE4: "Water volume setting 2",
            0xE5: "Washing time setting",
            0xE6: "“Number of times of rinsing” setting",
            0xE7: "Rinsing process setting",
            0xE8: "Spin drying time setting",
            0xE9: "Drying time setting",
            0xEA: "Warm water setting",
            0xEB: "Bathtub water recycle setting",
            0xEC: "Wrinkling minimization setting",
            0xED: "Time remaining to complete washer and dryer cycle",
            0xEE: "Door/cover lock setting",
            0xEF: "Washer and dryer cycle",
            0x90: "ON timer reservation setting",
            0x91: "ON timer setting",
            0x92: "Relative time-based ON timer setting",
        },
        0xD4: {  # Commercial show case outdoor unit class
            0x80: "Operation status",
            0xAA: "Exceptional status",
            0xB0: "Operating mode",
            0xBE: "Outdoor air temperature measurement",
            0xE2: "Compressor operation status",
            0xCA: "Group information",
        },
    },
    0x04: {  # Health-related device class group
        0x01: {  # Weighing machine class
            0x80: "Operation status",
            0xE0: "Measured value of body weight",
            0xE1: "Measured value of body fat",
        }
    },
    0x05: {  # Management/control-related device class group
        0xFD: {  # Switch (supporting JEM-A/HA terminals) class
            0x80: "Operation status",
            0xE0: "Connected device",
        }
    },
    0x06: {  # AV-related device class group
        0x01: {  # Display class
            0x80: "Operation status",
            0xB0: "Display control setting",
            0xB1: "Character string setting acceptance status",
            0xB2: "Supported character codes",
            0xB3: "Character string to present to the user",
            0xB4: "Length of character string accepted ",
        },
        0x02: {  # Television class
            0x80: "Operation status",
            0xB0: "Display control setting",
            0xB1: "Character string setting acceptance status",
            0xB2: "Supported character codes",
            0xB3: "Character string to present to the user",
            0xB4: "Length of character string accepted",
        },
        0x03: {  # Audio class
            0x80: "Operation status",
            0xB8: "Volume setting",
            0xB9: "Mute setting",
            0xBC: "Input source setting",
        },
        0x04: {  # Network camera class
            0x80: "Operation status",
            0xC0: "Still image photography setting acceptance status",
            0xC1: "Still image photography setting",
            0xD0: "Transfer setting",
        },
    },
    0x0E: {  # Profile class group
        0xF0: {  # Node profile class
            0x80: "Operating status",
            0x82: "Version information",
            0x83: "Identification number",
            0x89: "Fault content",
            0xBF: "Unique identifier data",
            0xD3: "Number of self-node instances",
            0xD4: "Number of self-node classes",
            0xD5: "Instance list notification",
            0xD6: "Self-node instance list S",
            0xD7: "Self-node class list S",
            # Super Class of the Profile class group
            0x88: "Fault status",
            0x8A: "Manufacturer code",
            0x8B: "Business facility code",
            0x8C: "Product code",
            0x8D: "Production number ",
            0x8E: "Production date",
            0x9D: "Status change announcement property map",
            0x9E: "Set property map",
            0x9F: "Get property map",
        }
    },
}

EPC_SUPER = {
    0x80: "Operation status",
    0x81: "Installation location",
    0x82: "Standard version information",
    0x83: "Identification number",
    0x84: "Measured instantaneous power consumption",
    0x85: "Measured cumulative power consumption",
    0x86: "Manufacturers fault code",
    0x87: "Current limit setting",
    0x88: "Fault status",
    0x89: "Fault description",
    0x8A: "Manufacturer code",
    0x8B: "Business facility code",
    0x8C: "Product code",
    0x8D: "Production number",
    0x8E: "Production date",
    0x8F: "Power-saving operation setting",
    0x93: "Remote control setting",
    0x97: "Current time setting",
    0x98: "Current date setting",
    0x99: "Power limit setting",
    0x9A: "Cumulative operating time",
    0x9B: "SetM property map",
    0x9C: "GetM property map",
    0x9D: "Status change announcement property map",
    0x9E: "Set property map",
    0x9F: "Get property map",
}



from .const import EHD1, EHD2, ESV_CODES
from .eojx import EOJX_CLASS, EOJX_GROUP


class TIDError(Exception):
    """Base class for other exceptions"""

    pass


def decodeEchonetMsg(byte):
    data = {}
    try:
        data["EHD1"] = byte[0]
        if data["EHD1"] not in EHD1:
            raise ValueError("EHD1 Header invalid")
        data["EHD2"] = byte[1]
        if data["EHD2"] not in EHD2:
            raise ValueError("EHD2 Header invalid")
        data["TID"] = int.from_bytes(byte[2:4], byteorder="big")

        # Decode SEOJ
        data["SEOJGC"] = byte[4]
        data["SEOJCC"] = byte[5]
        data["SEOJCI"] = byte[6]
        # Decode DEOJ
        data["DEOJGC"] = byte[7]
        data["DEOJCC"] = byte[8]
        data["DEOJCI"] = byte[9]

        # Decode Service property
        data["ESV"] = byte[10]

        i = 0
        epc_pointer = 12
        data["OPC"] = []
        # decode multiple processing properties (OPC)
        while i < (byte[11]):
            OPC = {}
            pdc_pointer = epc_pointer + 1
            edt_pointer = pdc_pointer + 1
            end_pointer = edt_pointer
            OPC["EPC"] = byte[epc_pointer]
            OPC["PDC"] = byte[pdc_pointer]
            pdc_len = byte[pdc_pointer]
            end_pointer += pdc_len
            OPC["EDT"] = byte[edt_pointer:end_pointer]
            epc_pointer = end_pointer
            i += 1
            data["OPC"].append(OPC)

    except ValueError as error:
        print("Caught this error: " + repr(error))
        quit()
    return data


def buildEchonetMsg(data):
    # EHD is fixed to 0x1081 because I am lazy.
    message = 0x1081

    # validate TID (set a default value if none provided)
    # TODO - TID message overlap.
    if "TID" not in data:
        data["TID"] = 0x0001
    elif data["TID"] > 0xFFFF:
        raise TIDError("Transaction ID is larger then 2 bytes.")
    message = (message << 16) + data["TID"]

    # append SEOJ which again is a fixed value to be lazy
    message = (message << 24) + 0x05FF01

    # validate DEOJ
    if data["DEOJGC"] in EOJX_GROUP:
        message = (message << 8) + data["DEOJGC"]
    else:
        raise ValueError(
            "Value " + str(hex(data["DEOJGC"])) + " not a valid SEO Group code"
        )

    if data["DEOJCC"] in EOJX_CLASS[data["DEOJGC"]]:
        message = (message << 8) + data["DEOJCC"]
    else:
        raise ValueError(
            "Value " + str(hex(data["DEOJCC"])) + " missing from SEO Class table. Check current ECHONETlite specs for any update."
        )

    message = (message << 8) + data["DEOJCI"]

    # validate ESV by looking up the codes.
    if data["ESV"] in ESV_CODES:
        # ESV code is a string
        message = (message << 8) + data["ESV"]
    else:
        raise ValueError("Value not in ESV code table")

    # validate OPC
    message = (message << 8) + len(data["OPC"])

    # You can have multiple OPC per transaction.
    for values in data["OPC"]:
        # validate EPC
        message = (message << 8) + values["EPC"]
        if "PDC" in values:
            message = (message << 8) + values["PDC"]
            # if PDC has a value then concat EDT to message
            if values["PDC"] > 0:
                message = (message << 8 * values["PDC"]) + values["EDT"]
        else:
            message = (message << 8) + 0x00
    # print(message)
    return bytearray.fromhex(format(int(message), "x"))


def preparePayload(tid, deojgc, deojcc, deojci, esv, opc):
    tx_payload = {
        "TID": tid,  # Transaction ID 1
        "DEOJGC": deojgc,
        "DEOJCC": deojcc,
        "DEOJCI": deojci,
        "ESV": esv,
        "OPC": opc,
    }
    return tx_payload



from pychonet.EchonetInstance import EchonetInstance
from pychonet.lib.epc_functions import _int, _signed_int

MODES = {
    "auto": 0x41,
    "cool": 0x42,
    "heat": 0x43,
    "dry": 0x44,
    "fan_only": 0x45,
    "other": 0x40,
    "off": 0xFF,
}

FAN_SPEED = {
    "auto": 0x41,
    "minimum": 0x31,
    "low": 0x32,
    "medium-low": 0x33,
    "medium": 0x34,
    "medium-high": 0x35,
    "high": 0x36,
    "very-high": 0x37,
    "max": 0x38,
}

AIRFLOW_HORIZ = {
    "rc-right": 0x41,
    "left-lc": 0x42,
    "lc-center-rc": 0x43,
    "left-lc-rc-right": 0x44,
    "right": 0x51,
    "rc": 0x52,
    "center": 0x54,
    "center-right": 0x55,
    "center-rc": 0x56,
    "center-rc-right": 0x57,
    "lc": 0x58,
    "lc-right": 0x59,
    "lc-rc": 0x5A,
    "left": 0x60,
    "left-right": 0x61,
    "left-rc": 0x62,
    "left-rc-right": 0x63,
    "left-center": 0x64,
    "left-center-right": 0x65,
    "left-center-rc": 0x66,
    "left-center-rc-right": 0x67,
    "left-lc-right": 0x69,
    "left-lc-rc": 0x6A,
}

AIRFLOW_VERT = {
    "upper": 0x41,
    "upper-central": 0x44,
    "central": 0x43,
    "lower-central": 0x45,
    "lower": 0x42,
}

AUTO_DIRECTION = {"auto": 0x41, "non-auto": 0x42, "auto-vert": 0x43, "auto-horiz": 0x44}

SWING_MODE = {"not-used": 0x31, "vert": 0x41, "horiz": 0x42, "vert-horiz": 0x43}

ENL_STATUS = 0x80
ENL_FANSPEED = 0xA0
ENL_AUTO_DIRECTION = 0xA1
ENL_SWING_MODE = 0xA3
ENL_AIR_VERT = 0xA4
ENL_AIR_HORZ = 0xA5
ENL_HVAC_MODE = 0xB0
ENL_HVAC_SET_TEMP = 0xB3
ENL_HVAC_ROOM_HUMIDITY = 0xBA
ENL_HVAC_ROOM_TEMP = 0xBB
ENL_HVAC_OUT_TEMP = 0xBE

# ----- Home Air conditioner functions -------


def _0130A0(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x41: "auto",
        0x31: "minimum",
        0x32: "low",
        0x33: "medium-low",
        0x34: "medium",
        0x35: "medium-high",
        0x36: "high",
        0x37: "very-high",
        0x38: "max",
    }
    return values.get(op_mode, "Invalid setting")


# Automatic control of air flow direction setting
def _0130A1(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x41: "auto", 0x42: "non-auto", 0x43: "auto-vert", 0x44: "auto-horiz"}
    return values.get(op_mode, "Invalid setting")


def _0130AA(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x40: "Normal operation",
        0x41: "Defrosting",
        0x42: "Preheating",
        0x43: "Heat removal",
    }
    return values.get(op_mode, "invalid_setting")


# Automatic swing of air flow direction setting
def _0130A3(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {0x31: "not-used", 0x41: "vert", 0x42: "horiz", 0x43: "vert-horiz"}
    return values.get(op_mode, "invalid_setting")


# Air flow direction (vertical) setting
def _0130A4(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x41: "upper",
        0x44: "upper-central",
        0x43: "central",
        0x45: "lower-central",
        0x42: "lower",
    }
    return values.get(op_mode, "invalid_setting")


# Air flow direction (horiziontal) setting
def _0130A5(edt):
    # complies with version 2.01 Release a (page 3-88)
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x41: "rc-right",
        0x42: "left-lc",
        0x43: "lc-center-rc",
        0x44: "left-lc-rc-right",
        0x51: "right",
        0x52: "rc",
        0x54: "center",
        0x55: "center-right",
        0x56: "center-rc",
        0x57: "center-rc-right",
        0x58: "lc",
        0x59: "lc-right",
        0x5A: "lc-rc",
        0x60: "left",
        0x61: "left-right",
        0x62: "left-rc",
        0x63: "left-rc-right",
        0x64: "left-center",
        0x65: "left-center-right",
        0x66: "left-center-rc",
        0x67: "left-center-rc-right",
        0x69: "left-lc-right",
        0x6A: "left-lc-rc",
    }
    return values.get(op_mode, "invalid_setting")


# Operation mode
def _0130B0(edt):
    op_mode = int.from_bytes(edt, "big")
    values = {
        0x41: "auto",
        0x42: "cool",
        0x43: "heat",
        0x44: "dry",
        0x45: "fan_only",
        0x40: "other",
    }
    return values.get(op_mode, "invalid_setting")


class HomeAirConditioner(EchonetInstance):

    EPC_FUNCTIONS = {
        0xA0: _0130A0,
        0xA1: _0130A1,
        0xA3: _0130A3,
        0xA4: _0130A4,
        0xA5: _0130A5,
        0xAA: _0130AA,
        0xB0: _0130B0,
        0xBA: _int,
        0xB3: _signed_int,
        0xBB: _signed_int,
        0xBE: _signed_int,
    }

    """
    Construct a new 'HomeAirConditioner' object.
    In theory this would work for any ECHONET enabled domestic AC.

    :param instance: Instance ID
    :param netif: IP address of node
    """

    def __init__(self, host, api_connector=None, instance=0x1):
        self._eojgc = 0x01  # Air conditioner-related device group
        self._eojcc = 0x30  # Home air conditioner class
        EchonetInstance.__init__(
            self, host, self._eojgc, self._eojcc, instance, api_connector
        )

    """
    getOperationalTemperature get the temperature that has been set in the HVAC

    return: A string representing the configured temperature.
    """

    def getOperationalTemperature(self):
        return self.getMessage(ENL_HVAC_SET_TEMP)

    """
    getRoomHumidity get the HVAC's room humidity.

    return: A integer representing the room humidity.
    """

    def getRoomHumidity(self):
        return self.getMessage(ENL_HVAC_ROOM_HUMIDITY)

    """
    getRoomTemperature get the HVAC's room temperature.

    return: A integer representing the room temperature.
    """

    def getRoomTemperature(self):
        return self.getMessage(ENL_HVAC_ROOM_TEMP)

    """
    getOutdoorTemperature get the outdoor temperature that has been set in the HVAC

    return: An integer representing the configured outdoor temperature.
    """

    def getOutdoorTemperature(self):
        return self.getMessage(ENL_HVAC_OUT_TEMP)

    """
    setOperationalTemperature get the temperature that has been set in the HVAC

    param temperature: A string representing the desired temperature.
    """

    def setOperationalTemperature(self, temperature):
        return self.setMessage(ENL_HVAC_SET_TEMP, int(temperature))

    """
    GetMode returns the current configured mode (e.g Heating, Cooling, Fan etc)

    return: A string representing the configured mode.
    """

    def getMode(self):
        return self.getMessage(ENL_HVAC_MODE)

    """
    setMode set the desired mode (e.g Heating, Cooling, Fan etc)
    Home Assistant compatabile with 'off' as valid option.
    If HVAC is OFF, setting a mode will switch it on.

    param mode: A string representing the desired mode.
    """

    def setMode(self, mode):
        if mode == "off":
            return self.setMessage(ENL_STATUS, 0x31)
        return self.setMessages(
            [
                {"EPC": ENL_STATUS, "PDC": 0x01, "EDT": 0x30},
                {"EPC": ENL_HVAC_MODE, "PDC": 0x01, "EDT": MODES[mode]},
            ]
        )

    """
    GetFanSpeed gets the current fan speed (e.g Low, Medium, High etc)
    Refer EPC code 0xA0: ('Air flow rate setting')

    return: A string representing the fan speed
    """

    def getFanSpeed(self):  # 0xA0
        return self.getMessage(ENL_FANSPEED)

    """
    setFanSpeed set the desired fan speed (e.g Low, Medium, High etc)

    param fans_speed: A string representing the fan speed
    """

    def setFanSpeed(self, fan_speed):
        return self.setMessage(ENL_FANSPEED, FAN_SPEED[fan_speed])

    """
    setSwingMode sets the automatic swing mode function

    params swing_mode: A string representing automatic swing mode
                       e.g: 'not-used', 'vert', 'horiz', 'vert-horiz'
    """

    def setSwingMode(self, swing_mode):
        return self.setMessage(ENL_SWING_MODE, SWING_MODE[swing_mode])

    """
    getSwingMode gets the swing mode that has been set in the HVAC

    return: A string representing the configured swing mode.
    """

    def getSwingMode(self):  # 0xA3
        return self.getMessage(ENL_SWING_MODE)

    """
    setAutoDirection sets the automatic direction mode function

    params auto_direction: A string representing automatic direction mode
                           e.g: 'auto', 'non-auto', 'auto-horiz', 'auto-vert'
    """

    def setAutoDirection(self, auto_direction):
        return self.setMessage(ENL_AUTO_DIRECTION, AUTO_DIRECTION[auto_direction])

    """
    getAutoDirection get the direction mode that has been set in the HVAC

    return: A string representing the configured temperature.
    """

    def getAutoDirection(self):  # 0xA1
        return self.getMessage(ENL_AUTO_DIRECTION)

    """
    setAirflowVert sets the vertical vane setting

    params airflow_vert: A string representing vertical airflow setting
                         e.g: 'upper', 'upper-central', 'central',
                         'lower-central', 'lower'
    """

    def setAirflowVert(self, airflow_vert):
        return self.setMessage(ENL_AIR_VERT, AIRFLOW_VERT[airflow_vert])

    """
    getAirflowVert get the vertical vane setting that has been set in the HVAC

    return: A string representing vertical airflow setting
    """

    def getAirflowVert(self):  # 0xA4
        return self.getMessage(ENL_AIR_VERT)

    """
    setAirflowHoriz sets the horizontal vane setting

    params airflow_horiz: A string representing horizontal airflow setting
                          e.g: 'left', 'lc', 'center', 'rc', 'right'
    """

    def setAirflowHoriz(self, airflow_horiz):
        return self.setMessage(ENL_AIR_HORZ, AIRFLOW_HORIZ[airflow_horiz])

    """
    getAirflowHoriz get the horizontal vane setting that has been set in the HVAC

    return: A string representing vertical airflow setting e.g: 'left', 'lc', 'center', 'rc', 'right'
    """

    def getAirflowHoriz(self):  # 0xA5
        return self.getMessage(ENL_AIR_HORZ)



# read the contents of your README file
from os import path

from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md")) as f:
    long_description = f.read()

setup(
    name="pychonet",
    version="2.2.0",
    author="Scott Phillips",
    author_email="scotty.phillips@hotmail.com",
    packages=["pychonet", "pychonet.lib"],
    url="http://pypi.python.org/pypi/pychonet/",
    license="LICENSE.txt",
    description="A library for interfacing via the ECHONETlite protocol.",
    long_description=long_description,
    install_requires=["aio-udp-server"],
    long_description_content_type="text/markdown",
)



from pychonet.lib.const import (ENL_CUMULATIVE_POWER, ENL_CUMULATIVE_RUNTIME,
                                ENL_GETMAP, ENL_INSTANTANEOUS_POWER,
                                ENL_MANUFACTURER, ENL_OFF, ENL_ON, ENL_SETMAP,
                                ENL_STATUS, ENL_UID, GET, SETC)
from pychonet.lib.epc import EPC_CODE, EPC_SUPER
from pychonet.lib.epc_functions import EPC_SUPER_FUNCTIONS

"""
Superclass for Echonet instance objects.
"""


class EchonetInstance:

    EPC_FUNCTIONS = {}
    """
    Constructs an object to represent an Echonet lite instance .

    :param eojgc: Echonet group code
    :param eojcc: Echonet class code
    :param instance: Instance ID
    :param netif: IP address of node
    """

    def __init__(self, host, eojgc, eojcc, instance, api_connector=None):
        self._host = host
        self._eojgc = eojgc
        self._eojcc = eojcc
        self._eojci = instance
        self._api = api_connector

        # TODO self instntiate the API connector for backwards compatability with the older libary

    """
    getMessage is used to fire a single ECHONET get messages to get Node information
    Assumes one EPC is sent per message.

    :param tx_epc: EPC byte code for the request.
    :return: the deconstructed payload for the response

    """

    async def getMessage(self, epc, pdc=0x00):
        opc = [{"EPC": epc, "PDC": pdc}]
        response = await self._api.echonetMessage(
            self._host, self._eojgc, self._eojcc, self._eojci, GET, opc
        )
        if not response:
            return False
        edt = self._api._state[self._host]["instances"][self._eojgc][self._eojcc][
            self._eojci
        ][epc]
        return edt

    """
    setMessage is used to fire ECHONET set messages to set Node information
    Assumes one OPC is sent per message.

    :param tx_epc: EPC byte code for the request.
    :param tx_edt: EDT data relevant to the request.
    :return: True if sucessful, false if request message failed
    """

    async def setMessage(self, epc, edt, pdc=0x01):
        opc = [{"EPC": epc, "PDC": pdc, "EDT": edt}]
        response = await self._api.echonetMessage(
            self._host, self._eojgc, self._eojcc, self._eojci, SETC, opc
        )
        if not response:
            return False
        return True

    """
    setMessage is used to fire ECHONET set messages to set Node information
    Assumes one OPC is sent per message.

    :param tx_epc: EPC byte code for the request.
    :param tx_edt: EDT data relevant to the request.
    :return: True if sucessful, false if request message failed
    """

    async def setMessages(self, opc):
        response = await self._api.echonetMessage(
            self._host, self._eojgc, self._eojcc, self._eojci, SETC, opc
        )
        if not response:
            return False
        return True

    """
    update is used as a way of producing a dict useful for API polling etc
    Data will either be formatted if you are using a specific class (e.g HomeAirConditioner)
    or returned as hex string values if you are using EchonetInstance

    :param attributes: optional list of EPC codes. eg [0x80, 0xBF], or a single code eg 0x80

    :return dict: A dict with the following attributes:
    {128: 'On', 160: 'medium-high', 176: 'heat', 129: '00', 130: '00004300',
    131: '0000060104a0c9a0fffe069719013001', 179: 19, 134: '06000006000000020000', 136: '42', 137: '0000'}

    :return string: if attribute is a single code then return value directly:
    eg:
    update(0x80)
    'on'

    """

    async def update(self, attributes=None):
        opc = []
        if attributes is None:
            attributes = self.getGetProperties()
        if isinstance(attributes, int):
            list_attributes = [attributes]
            attributes = list_attributes
        returned_json_data = {}
        for value in attributes:
            if value in self.getGetProperties():
                opc.append({"EPC": value})
        response = await self._api.echonetMessage(
            self._host, self._eojgc, self._eojcc, self._eojci, GET, opc
        )
        if response is not False:
            for epc in attributes:
                if epc not in list(
                    self._api._state[self._host]["instances"][self._eojgc][self._eojcc][
                        self._eojci
                    ].keys()
                ):
                    returned_json_data.update({epc: False})
                    continue
                elif epc in list(
                    EPC_SUPER_FUNCTIONS.keys()
                ):  # check if function is defined in the superset
                    returned_json_data.update(
                        {
                            epc: EPC_SUPER_FUNCTIONS[epc](
                                self._api._state[self._host]["instances"][self._eojgc][
                                    self._eojcc
                                ][self._eojci][epc]
                            )
                        }
                    )
                    continue
                elif epc in list(
                    EPC_SUPER.keys()
                ):  # return hex value if code exists in superset but no function found
                    returned_json_data.update(
                        {
                            epc: self._api._state[self._host]["instances"][self._eojgc][
                                self._eojcc
                            ][self._eojci][epc].hex()
                        }
                    )
                    continue
                elif epc in list(
                    self.EPC_FUNCTIONS.keys()
                ):  # check the class-specific EPC function table.
                    returned_json_data.update(
                        {
                            epc: self.EPC_FUNCTIONS[epc](
                                self._api._state[self._host]["instances"][self._eojgc][
                                    self._eojcc
                                ][self._eojci][epc]
                            )
                        }
                    )
                    continue
                elif epc in list(
                    EPC_CODE[self._eojgc][self._eojcc].keys()
                ):  # return hex value if EPC code exists in class but no function found
                    returned_json_data.update(
                        {
                            epc: self._api._state[self._host]["instances"][self._eojgc][
                                self._eojcc
                            ][self._eojci][epc].hex()
                        }
                    )
        for epc in attributes:
            if epc not in list(returned_json_data.keys()):
                returned_json_data.update({epc: False})
        if (len(returned_json_data)) == 1 and len(attributes) == 1:
            return returned_json_data[attributes[0]]
        elif (len(returned_json_data)) == 0:
            return False
        return returned_json_data

    """
    getIdentificationNumber returns a number used to identify an object uniquely

    :return: Identification number as a string.
    """

    async def getIdentificationNumber(self):  # EPC 0x83
        await self._api.getIdentificationNumber(
            self._host, self._eojgc, self._eojcc, self._eojci
        )
        return self._api._status[self._host]["instances"][self._eojgc][self._eojcc][
            self._eojci
        ][ENL_UID]

    """
    getManufacturer returns the manufacturer name if mapped, or Id otherwise

    :return: Manufacturer name as a string or echonet identification number as an int.
    """

    async def getManufacturer(self):  # EPC 0x8A
        return await self.update(ENL_MANUFACTURER)

    """
    getInstantaneousPower returns the current number of Wh the node is consuming.
    :return: Instantaneous Wh as an integer.
    """

    async def getInstantaneousPower(self):  # EPC 0x84
        return self.update(ENL_INSTANTANEOUS_POWER)

    """
    getCumulativePower returns the total number of Wh the node has used.
    Value should always increment up to 999,999,999 Wh after which resets to 0.
    :return: Cumulative Wh as an integer.
    """

    async def getCumulativePower(self):  # EPC 0x85
        return self.update(ENL_CUMULATIVE_POWER)

    """
    getCumulativeRuntime returns the total number of seconds the node has been running.
    :return: Runtime in seconds as an integer.
    """

    async def getCumulativeRuntime(self):  # EPC 0x9A
        return await self.update(ENL_CUMULATIVE_RUNTIME)

    """
    getOperationalStatus returns the ON/OFF state of the node

    :return: status as a string.
    """

    async def getOperationalStatus(self):  # EPC 0x80
        return await self.getMessage(ENL_STATUS)

    """
    On sets the node to ON.

    """

    async def on(self):  # EPC 0x80
        return await self.setMessage(ENL_STATUS, ENL_ON)

    """
    Off sets the node to OFF.

    """

    async def off(self):  # EPC 0x80
        return await self.setMessage(ENL_STATUS, ENL_OFF)

    def getSetProperties(self):  # EPC 0x9E
        return self._api._state[self._host]["instances"][self._eojgc][self._eojcc][
            self._eojci
        ][ENL_SETMAP]

    def getGetProperties(self):  # EPC 0x9F
        return self._api._state[self._host]["instances"][self._eojgc][self._eojcc][
            self._eojci
        ][ENL_GETMAP]

    async def getAllPropertyMaps(self):
        return await self._api.getAllPropertyMaps(
            self._host, self._eojgc, self._eojcc, self._eojci
        )
